import unittest

from AccessControl import Unauthorized


def addPythonScript(folder, id, params='', body=''):
    """Add a PythonScript to folder."""
    from Products.PythonScripts.PythonScript import manage_addPythonScript
    # clean up any 'ps' that's already here..
    if id in folder:
        del folder[id]
    manage_addPythonScript(folder, id)
    folder[id].ZPythonScript_edit(params, body)


def checkRestricted(folder, psbody):
    """Perform a check by running restricted Python code."""
    addPythonScript(folder, 'ps', body=psbody)
    try:
        folder.ps()
    except Unauthorized, e:
        raise AssertionError(e)


def checkUnauthorized(folder, psbody):
    """Perform a check by running restricted Python code.  Expect to
    encounter an Unauthorized exception."""
    addPythonScript(folder, 'ps', body=psbody)
    try:
        folder.ps()
    except Unauthorized:
        pass
    else:
        raise AssertionError("Authorized but shouldn't be")


def test_resource_restricted_code():
    """
    Set up the test fixtures:

      >>> import Products.Five.browser.tests
      >>> from Zope2.App import zcml
      >>> zcml.load_config("configure.zcml", Products.Five)
      >>> zcml.load_config('resource.zcml', package=Products.Five.browser.tests)

      >>> from Products.Five.tests.testing import manage_addFiveTraversableFolder
      >>> manage_addFiveTraversableFolder(self.folder, 'testoid', 'Testoid')

      >>> import os, glob
      >>> _prefix = os.path.dirname(Products.Five.browser.tests.__file__)
      >>> dir_resource_names = [os.path.basename(r) for r in (
      ...     glob.glob('%s/*.png' % _prefix) +
      ...     glob.glob('%s/*.pt' % _prefix) +
      ...     glob.glob('%s/[a-z]*.py' % _prefix) +
      ...     glob.glob('%s/*.css' % _prefix))]

      >>> from Products.Five.browser.tests.test_scriptsecurity import checkRestricted
      >>> from Products.Five.browser.tests.test_scriptsecurity import checkUnauthorized

      >>> resource_names = ['cockatiel.html', 'style.css', 'pattern.png']

    We should get Unauthorized as long as we're unauthenticated:

      >>> for resource in resource_names:
      ...     checkUnauthorized(
      ...         self.folder,
      ...         'context.restrictedTraverse("testoid/++resource++%s")()' % resource)

      >>> base = 'testoid/++resource++fivetest_resources/%s'
      >>> for resource in dir_resource_names:
      ...     path = base % resource
      ...     checkUnauthorized(self.folder, 'context.restrictedTraverse("%s")' % path)

    Now let's create a manager user account and log in:

      >>> uf = self.folder.acl_users
      >>> _ignored = uf._doAddUser('manager', 'r00t', ['Manager'], [])
      >>> self.login('manager')

    We can now view them all:

      >>> for resource in resource_names:
      ...     checkRestricted(
      ...         self.folder,
      ...         'context.restrictedTraverse("testoid/++resource++%s")()' % resource)

      >>> base = 'testoid/++resource++fivetest_resources/%s'
      >>> for resource in dir_resource_names:
      ...     path = base % resource
      ...     checkRestricted(self.folder, 'context.restrictedTraverse("%s")' % path)

    Let's make sure restrictedTraverse() works directly, too. It used to get
    tripped up on subdirectories due to missing security declarations.

      >>> self.folder.restrictedTraverse('++resource++fivetest_resources/resource.txt') is not None
      True
  
      >>> self.folder.restrictedTraverse('++resource++fivetest_resources/resource_subdir/resource.txt') is not None
      True

    Clean up

      >>> from zope.component.testing import tearDown
      >>> tearDown()
    """

def test_view_restricted_code():
    """
    Let's register a quite large amount of test pages:

      >>> import Products.Five.browser.tests
      >>> from Zope2.App import zcml
      >>> zcml.load_config("configure.zcml", Products.Five)
      >>> zcml.load_config('pages.zcml', package=Products.Five.browser.tests)

    Let's add a test object that we view most of the pages off of:

      >>> from Products.Five.tests.testing.simplecontent import manage_addSimpleContent
      >>> manage_addSimpleContent(self.folder, 'testoid', 'Testoid')

    We also need to create a stub user account and login; otherwise we
    wouldn't have all the rights to do traversal etc.:

      >>> uf = self.folder.acl_users
      >>> _ignored = uf._doAddUser('manager', 'r00t', ['Manager'], [])
      >>> self.login('manager')

      >>> protected_view_names = [
      ...     'eagle.txt', 'falcon.html', 'owl.html', 'flamingo.html',
      ...     'condor.html', 'permission_view']
      >>> 
      >>> public_view_names = [
      ...     'public_attribute_page',
      ...     'public_template_page',
      ...     'public_template_class_page',
      ...     'nodoc-method', 'nodoc-function', 'nodoc-object',
      ...     'dirpage1', 'dirpage2']

      >>> from Products.Five.browser.tests.test_scriptsecurity import checkRestricted
      >>> from Products.Five.browser.tests.test_scriptsecurity import checkUnauthorized

    As long as we're not authenticated, we should get Unauthorized for
    protected views, but we should be able to view the public ones:

      >>> self.logout()
      >>> for view_name in protected_view_names:
      ...     checkUnauthorized(
      ...         self.folder,
      ...         'context.restrictedTraverse("testoid/%s")()' % view_name)

      >>> for view_name in public_view_names:
      ...     checkRestricted(
      ...         self.folder,
      ...         'context.restrictedTraverse("testoid/%s")()' % view_name)
      >>> self.login('manager')

    Being logged in as a manager again, we find that the protected pages
    are accessible to us:

      >>> for view_name in protected_view_names:
      ...     checkRestricted(
      ...         self.folder,
      ...         'context.restrictedTraverse("testoid/%s")()' % view_name)

      >>> checkRestricted(
      ...     self.folder,
      ...     'context.restrictedTraverse("testoid/eagle.method").eagle()')

    Even when logged in though the private methods should not be accessible:

      >>> checkUnauthorized( self.folder,
      ...             'context.restrictedTraverse("testoid/eagle.method").mouse()')

    Cleanup:

      >>> from zope.component.testing import tearDown
      >>> tearDown()
    """


def test_suite():
    suite = unittest.TestSuite()
    try:
        import Products.PythonScripts
    except ImportError:
        pass
    else:
        from Testing.ZopeTestCase import ZopeDocTestSuite
        from Testing.ZopeTestCase import installProduct
        installProduct('PythonScripts')
        suite.addTest(ZopeDocTestSuite())
    return suite



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

from os.path import dirname

from AccessControl.class_init import InitializeClass
from AccessControl.SecurityInfo import ClassSecurityInfo
import App
from App.ImageFile import ImageFile
import HelpSys
import OFS
import webdav


class misc_:
    "Miscellaneous product information"
    security = ClassSecurityInfo()
    security.declareObjectPublic()

InitializeClass(misc_)


class p_:
    "Shared system information"
    security = ClassSecurityInfo()
    security.declareObjectPublic()

    here = dirname(__file__)
    broken = ImageFile('www/broken.gif', here)

    ofs_dir = dirname(OFS.__file__)
    User_icon = ImageFile('www/User_icon.gif', ofs_dir)

    locked = ImageFile('www/modified.gif', here)
    lockedo = ImageFile('www/locked.gif', here)

    davlocked = ImageFile('www/davlock.gif', dirname(webdav.__file__))

    pl = ImageFile('www/Plus_icon.gif', ofs_dir)
    mi = ImageFile('www/Minus_icon.gif', ofs_dir)

    app_dir = dirname(App.__file__)
    rtab = ImageFile('www/rtab.gif', app_dir)
    ltab = ImageFile('www/ltab.gif', app_dir)
    sp = ImageFile('www/sp.gif', app_dir)
    r_arrow_gif = ImageFile('www/r_arrow.gif', here)
    l_arrow_gif = ImageFile('www/l_arrow.gif', here)

    ControlPanel_icon = ImageFile('www/ControlPanel_icon.gif', ofs_dir)
    ApplicationManagement_icon = ImageFile('www/cpSystem.gif', app_dir)
    DatabaseManagement_icon = ImageFile('www/dbManage.gif', app_dir)
    DebugManager_icon = ImageFile('www/DebugManager_icon.gif', app_dir)
    InstalledProduct_icon = ImageFile('www/installedProduct.gif', app_dir)
    BrokenProduct_icon = ImageFile('www/brokenProduct.gif', app_dir)
    Product_icon = ImageFile('www/product.gif', app_dir)
    Permission_icon = ImageFile('www/permission.gif', app_dir)
    ProductFolder_icon = ImageFile('www/productFolder.gif', app_dir)
    PyPoweredSmall_Gif = ImageFile('www/PythonPoweredSmall.gif', app_dir)

    ZopeButton = ImageFile('www/zope_button.jpg', app_dir)
    ZButton = ImageFile('www/z_button.jpg', app_dir)
    zopelogo_jpg = ImageFile('www/zopelogo.jpg', app_dir)

    Properties_icon = ImageFile('www/Properties_icon.gif', ofs_dir)
    Propertysheets_icon = ImageFile('www/Properties_icon.gif', ofs_dir)

    helpsys_dir = dirname(HelpSys.__file__)
    ProductHelp_icon=ImageFile('images/productHelp.gif', helpsys_dir)
    HelpTopic_icon=ImageFile('images/helpTopic.gif', helpsys_dir)

InitializeClass(p_)


class Misc_:
    "Miscellaneous product information"
    security = ClassSecurityInfo()
    security.declareObjectPublic()

    def __init__(self, name, dict):
        self._d=dict
        self.__name__=name

    def __str__(self): return self.__name__
    def __getitem__(self, name): return self._d[name]
    def __setitem__(self, name, v): self._d[name]=v

InitializeClass(Misc_)



import unittest


class AccessRuleTests(unittest.TestCase):

    _old_SAR = None

    def setUp(self):
        from Testing.ZopeTestCase import ZopeLite
        ZopeLite.startup()

    def tearDown(self):
        if self._old_SAR is not None:
            self._set_SUPPRESS_ACCESSRULE(self._old_SAR)

    def _set_SUPPRESS_ACCESSRULE(self, value):
        from Products.SiteAccess import AccessRule as AR
        (self._old_SAR,
         AR.SUPPRESS_ACCESSRULE) = (AR.SUPPRESS_ACCESSRULE, value)

    def _getTargetClass(self):
        from Products.SiteAccess.AccessRule import AccessRule
        return AccessRule

    def _makeOne(self, method_id='testing'):
        return self._getTargetClass()(method_id)

    def test___call___w_SUPPRESS_ACCESSRULE_set(self):
        self._set_SUPPRESS_ACCESSRULE(1)
        _called = []
        def _func(*args):
            _called.append(args)
        rule = self._makeOne()
        request = DummyRequest(TraversalRequestNameStack=[])
        container = DummyContainer(testing=_func)
        rule(container, request)
        self.assertFalse(_called)

    def test___call___w_SUPPRESS_ACCESSRULE_in_URL(self):
        # This behavior changed in landing lp:142878.
        _called = []
        def _func(*args):
            _called.append(args)
        rule = self._makeOne()
        request = DummyRequest(TraversalRequestNameStack=
                                    ['_SUPPRESS_ACCESSRULE'])
        request.steps = []
        container = DummyContainer(testing=_func)
        rule(container, request)
        self.assertTrue(_called)
        self.assertEqual(request._virtual_root, None)

    def test___call___wo_SUPPRESS_ACCESSRULE(self):
        _called = []
        def _func(*args):
            _called.append(args)
        rule = self._makeOne()
        request = DummyRequest(TraversalRequestNameStack=[])
        request.steps = []
        container = DummyContainer(testing=_func)
        rule(container, request)
        self.assertTrue(_called)
        self.assertEqual(request._virtual_root, None)


class Test_manage_addAccessRule(unittest.TestCase):

    def _callFUT(self, container, method_id, REQUEST):
        from Products.SiteAccess.AccessRule import manage_addAccessRule
        return manage_addAccessRule(container, method_id, REQUEST)

    def test_no_method_id_no_existing_rules_no_request(self):
        container = DummyContainer()
        result = self._callFUT(container, None, None)
        self.assertTrue(result is None)
        self.assertFalse(container.__dict__)

    def test_no_method_id_no_existing_rules_w_request(self):
        container = DummyContainer()
        result = self._callFUT(container, None, {'URL1': 'http://example.com/'})
        self.assertTrue(isinstance(result, str))
        self.assertTrue('<TITLE>No Access Rule</TITLE>' in result)
        self.assertFalse(container.__dict__)

    def test_no_method_id_w_existing_rules_no_request(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        container = DummyContainer()
        old_rule = container.old_rule = DummyObject(name='old_rule',
                                                    icon='rule_icon.jpg')
        registerBeforeTraverse(container, old_rule, 'AccessRule')
        result = self._callFUT(container, None, None)
        self.assertTrue(result is None)
        self.assertFalse(container.__before_traverse__)
        self.assertFalse('icon' in old_rule.__dict__)

    def test_w_method_id_w_existing_rules_w_request_none(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        container = DummyContainer()
        old_rule = container.old_rule = DummyObject(name='old_rule',
                                                    icon='rule_icon.jpg')
        registerBeforeTraverse(container, old_rule, 'AccessRule')
        request = DummyRequest(URL1 = 'http://example.com/')
        request.form = {'none': '1'}
        result = self._callFUT(container, None, request)
        self.assertTrue(isinstance(result, str))
        self.assertTrue('<TITLE>No Access Rule</TITLE>' in result)
        self.assertFalse(container.__before_traverse__)
        self.assertFalse('icon' in old_rule.__dict__)

    def test_w_invalid_method_id_w_existing_rules_no_request(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        container = DummyContainer()
        old_rule = container.old_rule = DummyObject(name='old_rule',
                                                    icon='rule_icon.jpg')
        registerBeforeTraverse(container, old_rule, 'AccessRule')
        result = self._callFUT(container, 'nonesuch', None)
        self.assertTrue(result is None)
        self.assertTrue((99, 'AccessRule') in container.__before_traverse__)
        rule = container.__before_traverse__[(99, 'AccessRule')]
        self.assertEqual(rule.name, 'old_rule')
        self.assertEqual(old_rule.icon, 'rule_icon.jpg')

    def test_w_invalid_method_id_w_existing_rules_w_request(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        container = DummyContainer()
        old_rule = container.old_rule = DummyObject(name='old_rule',
                                                    icon='rule_icon.jpg')
        registerBeforeTraverse(container, old_rule, 'AccessRule')
        request = DummyRequest(URL1 = 'http://example.com/')
        request.form = {}
        result = self._callFUT(container, 'nonesuch', request)
        self.assertTrue(isinstance(result, str))
        self.assertTrue('<TITLE>Invalid Method Id</TITLE>' in result)
        self.assertTrue((99, 'AccessRule') in container.__before_traverse__)
        rule = container.__before_traverse__[(99, 'AccessRule')]
        self.assertEqual(rule.name, 'old_rule')
        self.assertEqual(old_rule.icon, 'rule_icon.jpg')

    def test_w_valid_method_id_w_existing_rules_no_request(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        container = DummyContainer()
        old_rule = container.old_rule = DummyObject(name='old_rule',
                                                    icon='rule_icon.jpg')
        new_rule = container.new_rule = DummyObject(name='new_rule')
        registerBeforeTraverse(container, old_rule, 'AccessRule')
        result = self._callFUT(container, 'new_rule', None)
        self.assertTrue(result is None)
        self.assertFalse((99, 'AccessRule') in container.__before_traverse__)
        self.assertTrue((1, 'AccessRule') in container.__before_traverse__)
        rule = container.__before_traverse__[(1, 'AccessRule')]
        self.assertEqual(rule.name, 'new_rule')
        self.assertFalse('icon' in old_rule.__dict__)
        self.assertEqual(new_rule.icon, 'misc_/SiteAccess/AccessRule.gif')

    def test_w_valid_method_id_w_existing_rules_w_request(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        container = DummyContainer()
        old_rule = container.old_rule = DummyObject(name='old_rule',
                                                    icon='rule_icon.jpg')
        new_rule = container.new_rule = DummyObject(name='new_rule')
        registerBeforeTraverse(container, old_rule, 'AccessRule')
        request = DummyRequest(URL1 = 'http://example.com/')
        request.form = {}
        result = self._callFUT(container, 'new_rule', request)
        self.assertTrue(isinstance(result, str))
        self.assertTrue('<TITLE>Access Rule Set</TITLE>' in result)
        self.assertFalse((99, 'AccessRule') in container.__before_traverse__)
        self.assertTrue((1, 'AccessRule') in container.__before_traverse__)
        rule = container.__before_traverse__[(1, 'AccessRule')]
        self.assertEqual(rule.name, 'new_rule')
        self.assertFalse('icon' in old_rule.__dict__)
        self.assertEqual(new_rule.icon, 'misc_/SiteAccess/AccessRule.gif')


class Test_getAccessRule(unittest.TestCase):

    def _callFUT(self, container, REQUEST=None):
        from Products.SiteAccess.AccessRule import getAccessRule
        return getAccessRule(container, REQUEST)

    def test_no_rules(self):
        container = DummyContainer()
        self.assertEqual(self._callFUT(container), '')

    def test_w_rule_invalid(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        container = DummyContainer()
        registerBeforeTraverse(container, DummyObject(), 'AccessRule')
        self.assertTrue(self._callFUT(container).startswith(
                                        'Invalid BeforeTraverse data: '))

    def test_w_rule_valid(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        container = DummyContainer()
        registerBeforeTraverse(container, DummyObject(name='foo'), 'AccessRule')
        self.assertEqual(self._callFUT(container), 'foo')


class DummyRequest(dict):
    _virtual_root = None
    def setVirtualRoot(self, root):
        self._virtual_root = root


class DummyObject(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)


class DummyContainer(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def this(self):
        return self

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(AccessRuleTests),
        unittest.makeSuite(Test_manage_addAccessRule),
        unittest.makeSuite(Test_getAccessRule),
    ))




# product initialisation
import ZReST
def initialize(context):
    context.registerClass(
        ZReST.ZReST,
        meta_type = 'ReStructuredText Document',
        icon='www/zrest.gif',
        constructors = (
            ZReST.manage_addZReSTForm, ZReST.manage_addZReST
        )
    )




##############################################################################
#
# Copyright (c) 2004, 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Five ZCML directive schemas
"""

from zope.interface import Interface
from zope.browserresource.metadirectives import IBasicResourceInformation
from zope.configuration.fields import GlobalObject, Tokens
from zope.schema import TextLine


# Deprecated, the class directive from zope.security allows the same
class IImplementsDirective(Interface):
    """State that a class implements something.
    """
    class_ = GlobalObject(
        title=u"Class",
        required=True
        )

    interface = Tokens(
        title=u"One or more interfaces",
        required=True,
        value_type=GlobalObject()
        )


class ISizableDirective(Interface):
    """Attach sizable adapters to classes.
    """

    class_ = GlobalObject(
        title=u"Class",
        required=True
        )


class IPagesFromDirectoryDirective(IBasicResourceInformation):
    """Register each file in a skin directory as a page resource
    """

    for_ = GlobalObject(
        title=u"The interface this view is for.",
        required=False
        )

    module = GlobalObject(
        title=u"Module",
        required=True
        )

    directory = TextLine(
        title=u"Directory",
        description=u"The directory containing the resource data.",
        required=True
        )


from zope.deferredimport import deprecated

deprecated("Please import from OFS.metadirectives",
    IRegisterPackageDirective = 'OFS.metadirectives:IRegisterPackageDirective',
    IRegisterClassDirective = 'OFS.metadirectives:IRegisterClassDirective',
    IDeprecatedManageAddDeleteDirective = \
        'OFS.metadirectives:IDeprecatedManageAddDeleteDirective',
)

deprecated("Please import from zope.configuration.xmlconfig",
    IInclude = 'zope.configuration.xmlconfig:IInclude',
)



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest

class WSGIResponseTests(unittest.TestCase):

    _old_NOW = None

    def tearDown(self):
        if self._old_NOW is not None:
            self._setNOW(self._old_NOW)

    def _getTargetClass(self):
        from ZPublisher.WSGIPublisher import WSGIResponse
        return WSGIResponse

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def _setNOW(self, value):
        from ZPublisher import WSGIPublisher
        WSGIPublisher._NOW, self._old_NOW = value, WSGIPublisher._NOW

    def test_finalize_sets_204_on_empty_not_streaming(self):
        response = self._makeOne()
        response.finalize()
        self.assertEqual(response.status, 204)

    def test_finalize_sets_204_on_empty_not_streaming_ignores_non_200(self):
        response = self._makeOne()
        response.setStatus(302)
        response.finalize()
        self.assertEqual(response.status, 302)

    def test_finalize_sets_content_length_if_missing(self):
        response = self._makeOne()
        response.setBody('TESTING')
        response.finalize()
        self.assertEqual(response.getHeader('Content-Length'), '7')

    def test_finalize_skips_setting_content_length_if_missing_w_streaming(self):
        response = self._makeOne()
        response._streaming = True
        response.body = 'TESTING'
        response.finalize()
        self.assertFalse(response.getHeader('Content-Length'))

    def test_finalize_HTTP_1_0_keep_alive_w_content_length(self):
        response = self._makeOne()
        response._http_version = '1.0'
        response._http_connection = 'keep-alive'
        response.setBody('TESTING')
        response.finalize()
        self.assertEqual(response.getHeader('Connection'), 'Keep-Alive')

    def test_finalize_HTTP_1_0_keep_alive_wo_content_length_streaming(self):
        response = self._makeOne()
        response._http_version = '1.0'
        response._http_connection = 'keep-alive'
        response._streaming = True
        response.finalize()
        self.assertEqual(response.getHeader('Connection'), 'close')

    def test_finalize_HTTP_1_0_not_keep_alive_w_content_length(self):
        response = self._makeOne()
        response._http_version = '1.0'
        response.setBody('TESTING')
        response.finalize()
        self.assertEqual(response.getHeader('Connection'), 'close')

    def test_finalize_HTTP_1_1_connection_close(self):
        response = self._makeOne()
        response._http_version = '1.1'
        response._http_connection = 'close'
        response.finalize()
        self.assertEqual(response.getHeader('Connection'), 'close')

    def test_finalize_HTTP_1_1_wo_content_length_streaming_wo_http_chunk(self):
        response = self._makeOne()
        response._http_version = '1.1'
        response._streaming = True
        response.http_chunk = 0
        response.finalize()
        self.assertEqual(response.getHeader('Connection'), 'close')
        self.assertEqual(response.getHeader('Transfer-Encoding'), None)
        self.assertFalse(response._chunking)

    def test_finalize_HTTP_1_1_wo_content_length_streaming_w_http_chunk(self):
        response = self._makeOne()
        response._http_version = '1.1'
        response._streaming = True
        response.http_chunk = 1
        response.finalize()
        self.assertEqual(response.getHeader('Connection'), None)

    def test_finalize_HTTP_1_1_w_content_length_wo_chunk_wo_streaming(self):
        response = self._makeOne()
        response._http_version = '1.1'
        response.setBody('TESTING')
        response.finalize()
        self.assertEqual(response.getHeader('Connection'), None)

    def test_listHeaders_skips_Server_header_wo_server_version_set(self):
        response = self._makeOne()
        response.setBody('TESTING')
        headers = response.listHeaders()
        sv = [x for x in headers if x[0] == 'Server']
        self.assertFalse(sv)

    def test_listHeaders_includes_Server_header_w_server_version_set(self):
        response = self._makeOne()
        response._server_version = 'TESTME'
        response.setBody('TESTING')
        headers = response.listHeaders()
        sv = [x for x in headers if x[0] == 'Server']
        self.assertTrue(('Server', 'TESTME') in sv)

    def test_listHeaders_includes_Date_header(self):
        import time
        WHEN = time.localtime()
        self._setNOW(time.mktime(WHEN))
        response = self._makeOne()
        response.setBody('TESTING')
        headers = response.listHeaders()
        whenstr = time.strftime('%a, %d %b %Y %H:%M:%S GMT',
                                time.gmtime(time.mktime(WHEN)))
        self.assertTrue(('Date', whenstr) in headers)

    def test_setBody_IUnboundStreamIterator(self):
        from ZPublisher.Iterators import IUnboundStreamIterator
        from zope.interface import implements

        class test_streamiterator:
            implements(IUnboundStreamIterator)
            data = "hello"
            done = 0

            def next(self):
                if not self.done:
                    self.done = 1
                    return self.data
                raise StopIteration

        response = self._makeOne()
        response.setStatus(200)
        body = test_streamiterator()
        response.setBody(body)
        response.finalize()
        self.assertTrue(body is response.body)
        self.assertEqual(response._streaming, 1)

    def test_setBody_IStreamIterator(self):
        from ZPublisher.Iterators import IStreamIterator
        from zope.interface import implements

        class test_streamiterator:
            implements(IStreamIterator)
            data = "hello"
            done = 0

            def next(self):
                if not self.done:
                    self.done = 1
                    return self.data
                raise StopIteration

            def __len__(self):
                return len(self.data)

        response = self._makeOne()
        response.setStatus(200)
        body = test_streamiterator()
        response.setBody(body)
        response.finalize()
        self.assertTrue(body is response.body)
        self.assertEqual(response._streaming, 0)
        self.assertEqual(response.getHeader('Content-Length'),
                         '%d' % len(test_streamiterator.data))

    #def test___str__already_wrote_not_chunking(self):
    #    response = self._makeOne()
    #    response._wrote = True
    #    response._chunking = False
    #    self.assertEqual(str(response), '')

    #def test___str__already_wrote_w_chunking(self):
    #    response = self._makeOne()
    #    response._wrote = True
    #    response._chunking = True
    #    self.assertEqual(str(response), '0\r\n\r\n')

    def test___str___raises(self):
        response = self._makeOne()
        response.setBody('TESTING')
        self.assertRaises(NotImplementedError, lambda: str(response))


class Test_publish(unittest.TestCase):

    def _callFUT(self, request, module_name, _get_module_info=None):
        from ZPublisher.WSGIPublisher import publish
        if _get_module_info is None:
            return publish(request, module_name)

        return publish(request, module_name, _get_module_info)

    def test_invalid_module_doesnt_catch_error(self):
        _gmi = DummyCallable()
        _gmi._raise = ImportError('testing')
        self.assertRaises(ImportError, self._callFUT, None, 'nonesuch', _gmi)
        self.assertEqual(_gmi._called_with, (('nonesuch',), {}))

    def test_wo_REMOTE_USER(self):
        request = DummyRequest(PATH_INFO='/')
        response = request.response = DummyResponse()
        _before = DummyCallable()
        _after = object()
        _object = DummyCallable()
        _object._result = 'RESULT'
        request._traverse_to = _object
        _realm = 'TESTING'
        _debug_mode = True
        _err_hook = DummyCallable()
        _validated_hook = object()
        _tm = DummyTM()
        _gmi = DummyCallable()
        _gmi._result = (_before, _after, _object, _realm, _debug_mode,
                        _err_hook, _validated_hook, _tm)
        returned = self._callFUT(request, 'okmodule', _gmi)
        self.assertTrue(returned is response)
        self.assertEqual(_gmi._called_with, (('okmodule',), {}))
        self.assertTrue(request._processedInputs)
        self.assertEqual(response.after_list, (_after,))
        self.assertTrue(response.debug_mode)
        self.assertEqual(response.realm, 'TESTING')
        self.assertEqual(_before._called_with, ((), {}))
        self.assertEqual(request['PARENTS'], [_object])
        self.assertEqual(request._traversed, ('/', None, _validated_hook))
        self.assertEqual(_tm._recorded, (_object, request))
        self.assertEqual(_object._called_with, ((), {}))
        self.assertEqual(response._body, 'RESULT')
        self.assertEqual(_err_hook._called_with, None)

    def test_w_REMOTE_USER(self):
        request = DummyRequest(PATH_INFO='/', REMOTE_USER='phred')
        response = request.response = DummyResponse()
        _before = DummyCallable()
        _after = object()
        _object = DummyCallable()
        _object._result = 'RESULT'
        request._traverse_to = _object
        _realm = 'TESTING'
        _debug_mode = True
        _err_hook = DummyCallable()
        _validated_hook = object()
        _tm = DummyTM()
        _gmi = DummyCallable()
        _gmi._result = (_before, _after, _object, _realm, _debug_mode,
                        _err_hook, _validated_hook, _tm)
        self._callFUT(request, 'okmodule', _gmi)
        self.assertEqual(response.realm, None)


class Test_publish_module(unittest.TestCase):
    
    def setUp(self):
        from zope.testing.cleanup import cleanUp

        cleanUp()

    def tearDown(self):
        from zope.testing.cleanup import cleanUp
        cleanUp()

    def _callFUT(self, environ, start_response,
                _publish=None, _response_factory=None, _request_factory=None):
        from ZPublisher.WSGIPublisher import publish_module
        if _publish is not None:
            if _response_factory is not None:
                if _request_factory is not None:
                    return publish_module(environ, start_response, _publish,
                                          _response_factory, _request_factory)
                return publish_module(environ, start_response, _publish,
                                      _response_factory)
            else:
                if _request_factory is not None:
                    return publish_module(environ, start_response, _publish,
                                        _request_factory=_request_factory)
                return publish_module(environ, start_response, _publish)
        return publish_module(environ, start_response)

    def _registerView(self, factory, name, provides=None):
        from zope.component import provideAdapter
        from zope.interface import Interface
        from zope.publisher.browser import IDefaultBrowserLayer
        from OFS.interfaces import IApplication
        if provides is None:
            provides = Interface
        requires = (IApplication, IDefaultBrowserLayer)
        provideAdapter(factory, requires, provides, name)

    def _makeEnviron(self, **kw):
        from StringIO import StringIO
        environ = {
            'SCRIPT_NAME' : '',
            'REQUEST_METHOD' : 'GET',
            'QUERY_STRING' : '',
            'SERVER_NAME' : '127.0.0.1',
            'REMOTE_ADDR': '127.0.0.1', 
            'wsgi.url_scheme': 'http', 
            'SERVER_PORT': '8080', 
            'HTTP_HOST': '127.0.0.1:8080', 
            'SERVER_PROTOCOL' : 'HTTP/1.1',
            'wsgi.input' : StringIO(''),
            'CONTENT_LENGTH': '0',
            'HTTP_CONNECTION': 'keep-alive',
            'CONTENT_TYPE': ''
        }
        environ.update(kw)
        return environ

    def test_calls_setDefaultSkin(self):
        from zope.traversing.interfaces import ITraversable
        from zope.traversing.namespace import view

        class TestView:
            __name__ = 'testing'

            def __init__(self, context, request):
                pass

            def __call__(self):
                return 'foobar'

        # Define the views
        self._registerView(TestView, 'testing')

        # Bind the 'view' namespace (for @@ traversal)
        self._registerView(view, 'view', ITraversable)

        environ = self._makeEnviron(PATH_INFO='/@@testing')
        self.assertEqual(self._callFUT(environ, noopStartResponse),
                         ('', 'foobar'))

    def test_publish_can_return_new_response(self):
        from ZPublisher.HTTPRequest import HTTPRequest
        _response = DummyResponse()
        _response.body = 'BODY'
        _after1 = DummyCallable()
        _after2 = DummyCallable()
        _response.after_list = (_after1, _after2)
        environ = self._makeEnviron()
        start_response = DummyCallable()
        _publish = DummyCallable()
        _publish._result = _response
        app_iter = self._callFUT(environ, start_response, _publish)
        self.assertEqual(app_iter, ('', 'BODY'))
        (status, headers), kw = start_response._called_with
        self.assertEqual(status, '204 No Content')
        self.assertEqual(headers, [('Content-Length', '0')])
        self.assertEqual(kw, {})
        (request, module), kw = _publish._called_with
        self.assertTrue(isinstance(request, HTTPRequest))
        self.assertEqual(module, 'Zope2')
        self.assertEqual(kw, {})
        self.assertTrue(_response._finalized)
        self.assertEqual(_after1._called_with, ((), {}))
        self.assertEqual(_after2._called_with, ((), {}))

    def test_swallows_Unauthorized(self):
        from zExceptions import Unauthorized
        environ = self._makeEnviron()
        start_response = DummyCallable()
        _publish = DummyCallable()
        _publish._raise = Unauthorized('TESTING')
        app_iter = self._callFUT(environ, start_response, _publish)
        self.assertEqual(app_iter, ('', ''))
        (status, headers), kw = start_response._called_with
        self.assertEqual(status, '401 Unauthorized')
        self.assertTrue(('Content-Length', '0') in headers)
        self.assertEqual(kw, {})

    def test_swallows_Redirect(self):
        from zExceptions import Redirect
        environ = self._makeEnviron()
        start_response = DummyCallable()
        _publish = DummyCallable()
        _publish._raise = Redirect('/redirect_to')
        app_iter = self._callFUT(environ, start_response, _publish)
        self.assertEqual(app_iter, ('', ''))
        (status, headers), kw = start_response._called_with
        self.assertEqual(status, '302 Moved Temporarily')
        self.assertTrue(('Location', '/redirect_to') in headers)
        self.assertTrue(('Content-Length', '0') in headers)
        self.assertEqual(kw, {})

    def test_response_body_is_file(self):
        class DummyFile(file):
            def __init__(self):
                pass
            def read(self, *args, **kw):
                raise NotImplementedError()
        _response = DummyResponse()
        _response._status = '200 OK'
        _response._headers = [('Content-Length', '4')]
        body = _response.body = DummyFile()
        environ = self._makeEnviron()
        start_response = DummyCallable()
        _publish = DummyCallable()
        _publish._result = _response
        app_iter = self._callFUT(environ, start_response, _publish)
        self.assertTrue(app_iter is body)

    def test_response_is_stream(self):
        from ZPublisher.Iterators import IStreamIterator
        from zope.interface import implements

        class test_streamiterator:
            implements(IStreamIterator)
            data = "hello"
            done = 0

            def next(self):
                if not self.done:
                    self.done = 1
                    return self.data
                raise StopIteration

        _response = DummyResponse()
        _response._status = '200 OK'
        _response._headers = [('Content-Length', '4')]
        body = _response.body = test_streamiterator()
        environ = self._makeEnviron()
        start_response = DummyCallable()
        _publish = DummyCallable()
        _publish._result = _response
        app_iter = self._callFUT(environ, start_response, _publish)
        self.assertTrue(app_iter is body)

    def test_response_is_unboundstream(self):
        from ZPublisher.Iterators import IUnboundStreamIterator
        from zope.interface import implements

        class test_unboundstreamiterator:
            implements(IUnboundStreamIterator)
            data = "hello"
            done = 0

            def next(self):
                if not self.done:
                    self.done = 1
                    return self.data
                raise StopIteration

        _response = DummyResponse()
        _response._status = '200 OK'
        body = _response.body = test_unboundstreamiterator()
        environ = self._makeEnviron()
        start_response = DummyCallable()
        _publish = DummyCallable()
        _publish._result = _response
        app_iter = self._callFUT(environ, start_response, _publish)
        self.assertTrue(app_iter is body)

    def test_request_closed_when_tm_middleware_not_active(self):
        environ = self._makeEnviron()
        start_response = DummyCallable()
        _request = DummyRequest()
        _request._closed = False
        def _close():
            _request._closed = True
        _request.close = _close
        def _request_factory(stdin, environ, response):
            return _request
        _publish = DummyCallable()
        _publish._result = DummyResponse()
        app_iter = self._callFUT(environ, start_response, _publish,
                                 _request_factory=_request_factory)
        self.assertTrue(_request._closed)

    def test_request_not_closed_when_tm_middleware_active(self):
        import transaction
        from ZPublisher import WSGIPublisher
        environ = self._makeEnviron()
        environ['repoze.tm.active'] = 1
        start_response = DummyCallable()
        _request = DummyRequest()
        _request._closed = False
        def _close():
            _request._closed = True
        _request.close = _close
        def _request_factory(stdin, environ, response):
            return _request
        _publish = DummyCallable()
        _publish._result = DummyResponse()
        app_iter = self._callFUT(environ, start_response, _publish,
                                 _request_factory=_request_factory)
        self.assertFalse(_request._closed)
        txn = transaction.get()
        self.assertTrue(txn in WSGIPublisher._request_closer_for_repoze_tm.requests)
        txn.commit()
        self.assertTrue(_request._closed)
        self.assertFalse(txn in WSGIPublisher._request_closer_for_repoze_tm.requests)
        # try again, but this time raise an exception and abort
        _request._closed = False
        _publish._raise = Exception('oops')
        self.assertRaises(Exception, self._callFUT,
                          environ, start_response, _publish,
                          _request_factory=_request_factory)
        self.assertFalse(_request._closed)
        txn = transaction.get()
        self.assertTrue(txn in WSGIPublisher._request_closer_for_repoze_tm.requests)
        txn.abort()
        self.assertFalse(txn in WSGIPublisher._request_closer_for_repoze_tm.requests)
        self.assertTrue(_request._closed)


class DummyRequest(dict):
    _processedInputs = False
    _traversed = None
    _traverse_to = None
    args = ()

    def processInputs(self):
        self._processedInputs = True

    def traverse(self, path, response=None, validated_hook=None):
        self._traversed = (path, response, validated_hook)
        return self._traverse_to

class DummyResponse(object):
    debug_mode = False
    after_list = ()
    realm = None
    _body = None
    _finalized = False
    _status = '204 No Content'
    _headers = [('Content-Length', '0')]

    def finalize(self):
        self._finalized = True
        return self._status, self._headers

    def setBody(self, body):
        self._body = body

    body = property(lambda self: self._body, setBody)

class DummyCallable(object):
    _called_with = _raise = _result = None

    def __call__(self, *args, **kw):
        self._called_with = (args, kw)
        if self._raise:
            raise self._raise
        return self._result

class DummyTM(object):
    _recorded = _raise = _result = None

    def recordMetaData(self, *args):
        self._recorded = args

def noopStartResponse(status, headers):
    pass


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(WSGIResponseTests),
        unittest.makeSuite(Test_publish),
        unittest.makeSuite(Test_publish_module),
    ))



#!python
##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""zopectl -- control Zope using zdaemon.

Usage: zopectl [options] [action [arguments]]

Options:
-h/--help -- print usage message and exit
-b/--backoff-limit SECONDS -- set backoff limit to SECONDS (default 10)
-d/--daemon -- run as a proper daemon; fork a subprocess, close files etc.
-f/--forever -- run forever (by default, exit when backoff limit is exceeded)
-h/--help -- print this usage message and exit
-i/--interactive -- start an interactive shell after executing commands
-l/--logfile -- log file to be read by logtail command
-u/--user -- run the daemon manager program as this user (or numeric id)
-m/--umask -- provide octal umask for files created by the managed process
-T/--start-timeout SECONDS -- Start timeout when a test program is used
-s/--socket-name -- socket between zopectl and zdrun

action [arguments] -- see below

Actions are commands like "start", "stop" and "status".  If -i is
specified or no action is specified on the command line, a "shell"
interpreting actions typed interactively is started (unless the
configuration option default_to_interactive is set to false).  Use the
action "help" to find out about available actions.
"""

import csv
import os
import sys
import signal

import pkg_resources

import zdaemon
import Zope2.Startup

from zdaemon.zdctl import ZDCmd
from zdaemon.zdoptions import ZDOptions
from ZConfig.components.logger.handlers import FileHandlerFactory
from ZConfig.datatypes import existing_dirpath

WIN = False
if sys.platform[:3].lower() == "win":
    WIN = True
    import pywintypes
    import win32service
    import win32serviceutil
    from nt_svcutils import service
    
    def do_windows(command):
        def inner(self,arg):

            name = self.get_service_name()
            display_name = 'Zope instance at '+self.options.directory

            # This class exists only so we can take advantage of
            # win32serviceutil.HandleCommandLine, it is never
            # instantiated.
            class InstanceService(service.Service):
                _svc_name_ = name
                _svc_display_name_ = display_name
                _svc_description_ = "A Zope application instance running as a service"

            # getopt sucks :-(
            argv = [sys.argv[0]]
            argv.extend(arg.split())
            argv.append(command)

            # we need to supply this manually as HandleCommandLine guesses wrong
            serviceClassName = os.path.splitext(service.__file__)[0]+'.Service'
            
            err = win32serviceutil.HandleCommandLine(
                InstanceService,
                serviceClassName,
                argv=argv,
                )

            self.InstanceClass = InstanceService
            return err
            
        return inner

def string_list(arg):
    return arg.split()

def quote_command(command):
    print " ".join(command)
    # Quote the program name, so it works even if it contains spaces
    command = " ".join(['"%s"' % x for x in command])
    if WIN:
        # odd, but true: the windows cmd processor can't handle more than
        # one quoted item per string unless you add quotes around the
        # whole line.
        command = '"%s"' % command
    return command

class ZopeCtlOptions(ZDOptions):
    # Zope controller options.
    # 
    # After initialization, this should look very much like a
    # zdaemon.zdctl.ZDCtlOptions instance.  Many of the attributes are
    # initialized from different sources, however.

    __doc__ = __doc__
    

    positional_args_allowed = 1
    schemadir = os.path.dirname(Zope2.Startup.__file__)
    schemafile = "zopeschema.xml"
    uid = gid = None

    # this indicates that no explict program has been provided.
    # the command line option can set this.
    program = None
    
    # this indicates that no explict socket name has been provided.
    # the command line option can set this.
    sockname = None
    
    # XXX Suppress using Zope's <eventlog> section to avoid using the
    # same logging for zdctl as for the Zope appserver.  There still
    # needs to be a way to set a logfile for zdctl.
    logsectionname = None
    
    def __init__(self):
        ZDOptions.__init__(self)
        self.add("program", "runner.program", "p:", "program=",
                 handler=string_list)
        self.add("backofflimit", "runner.backoff_limit",
                 "b:", "backoff-limit=", int, default=10)
        self.add("daemon", "runner.daemon", "d", "daemon", flag=1, default=1)
        self.add("forever", "runner.forever", "f", "forever",
                 flag=1, default=0)
        self.add("hang_around", "runner.hang_around", default=0)
        self.add("interactive", None, "i", "interactive", flag=1)
        self.add("default_to_interactive", "runner.default_to_interactive",
                 default=1)
        self.add("start_timeout", "runner.start_timeout",
                 "T:", "start-timeout=", int, default=300)
        self.add("logfile", None, "l:", "logfile=")
        self.add("user", "runner.user", "u:", "user=")
        self.add("prompt", "runner.prompt", default="zopectl>")
        self.add("umask", "runner.umask", "m:", "umask=")
        self.add("sockname", "runner.socket_name", "s:", "socket-name=",
                 existing_dirpath, default=None)

    def realize(self, *args, **kw):
        ZDOptions.realize(self, *args, **kw)
        # Additional checking of user option; set uid and gid
        if self.user is not None:
            import pwd
            try:
                uid = int(self.user)
            except ValueError:
                try:
                    pwrec = pwd.getpwnam(self.user)
                except KeyError:
                    self.usage("username %r not found" % self.user)
                uid = pwrec[2]
            else:
                try:
                    pwrec = pwd.getpwuid(uid)
                except KeyError:
                    self.usage("uid %r not found" % self.user)
            gid = pwrec[3]
            self.uid = uid
            self.gid = gid

        config = self.configroot
        self.directory = config.instancehome
        self.clienthome = config.clienthome
        if self.program:
            if isinstance(self.program, basestring):
                self.program = [self.program]
        elif config.runner and config.runner.program:
            self.program = config.runner.program
        else:
            self.program = [os.path.join(self.directory, "bin", "runzope")]
        if self.sockname:
            # set by command line option
            pass
        elif config.runner and config.runner.socket_name:
            self.sockname = config.runner.socket_name
        else:
            self.sockname = os.path.join(self.clienthome, "zopectlsock")
        self.python = os.environ.get('PYTHON', config.python) or sys.executable
        self.zdrun = os.path.join(os.path.dirname(zdaemon.__file__),
                                  "zdrun.py")

        self.exitcodes = [0, 2]
        if self.logfile is None and config.eventlog is not None:
            for handler in config.eventlog.handler_factories:
                if isinstance(handler, FileHandlerFactory):
                    self.logfile = handler.section.path
                    if self.logfile not in ("STDERR", "STDOUT"):
                        break


class ZopeCmd(ZDCmd):

    _exitstatus = 0

    def _get_override(self, opt, name, svalue=None, flag=0):
        # Suppress the config file, and pass all configuration via the
        # command line.  This avoids needing to specialize the zdrun
        # script.
        if name == "configfile":
            return []
        value = getattr(self.options, name)
        if value is None:
            return []
        if flag:
            if value:
                args = [opt]
            else:
                args = []
        else:
            if svalue is None:
                svalue = str(value)
            args = [opt, svalue]
        return args

    def do_start(self, arg):
        # signal to Zope that it is being managed
        # (to indicate it's web-restartable)
        os.putenv('ZMANAGED', '1')
        ZDCmd.do_start(self, arg)

    ## START OF WINDOWS ONLY STUFF
    
    if WIN:

        def get_service_name(self):
            return 'Zope'+str(hash(self.options.directory.lower()))
            
        def get_status(self):
            sn = self.get_service_name()
            try:
                stat = win32serviceutil.QueryServiceStatus(sn)[1]
                self.zd_up = 1
            except pywintypes.error, err:
                if err[0] == 1060:
                    # Service not installed
                    stat = win32service.SERVICE_STOPPED
                    self.zd_up = 0
                else:
                    raise

            self.zd_pid = (stat == win32service.SERVICE_RUNNING) and -1 or 0
            self.zd_status = "args=%s" % self.options.program
            
        do_start = do_windows('start')
        do_stop = do_windows('stop')
        do_restart = do_windows('restart')

        # Add extra commands to install and remove the Windows service

        def do_install(self,arg):
            err = do_windows('install')(self,arg)
            if not err:
                # If we installed successfully, put info in registry for the
                # real Service class to use:
                command = '"%s" -C "%s"' % (
                    # This gives us the instance script for buildout instances
                    # and the install script for classic instances.
                    os.path.join(os.path.split(sys.argv[0])[0],'runzope'),
                    self.options.configfile
                    )
                self.InstanceClass.setReg('command',command)
                
                # This is unfortunately needed because runzope.exe is a setuptools
                # generated .exe that spawns off a sub process, so pid would give us
                # the wrong event name.
                self.InstanceClass.setReg('pid_filename',self.options.configroot.pid_filename)
            return err

        def help_install(self):
            print "install -- Installs Zope as a Windows service."

        do_remove = do_windows('remove')

        def help_remove(self):
            print "remove -- Removes the Zope Windows service."

        do_windebug = do_windows('debug')
        
        def help_windebug(self):
            print "windebug -- Runs the Zope Windows service in the foreground, in debug mode."

    ## END OF WINDOWS ONLY STUFF
            
    def get_startup_cmd(self, python, more):
        cmdline = ( '%s -c "from Zope2 import configure;'
                    'configure(%r);' %
                    (python, self.options.configfile)
                    )
        return cmdline + more + '\"'

    def do_debug(self, arg):
        cmdline = self.get_startup_cmd(self.options.python + ' -i',
                                       'import Zope2; app=Zope2.app()')
        print ('Starting debugger (the name "app" is bound to the top-level '
               'Zope object)')
        os.system(cmdline)

    def do_foreground(self, arg):
        program = self.options.program
        local_additions = []
        if not program.count('-X'):
            local_additions += ['-X']
        if not program.count('debug-mode=on'):
            local_additions += ['debug-mode=on']
        program[1:1] = local_additions
        command = quote_command(program)
        try:
            return os.system(command)
        except KeyboardInterrupt:
            pass
        finally:
            for addition in local_additions: program.remove(addition)
            
    def help_debug(self):
        print "debug -- run the Zope debugger to inspect your database"
        print "         manually using a Python interactive shell"

    def __getattr__(self, name):
        """Getter to check if an unknown command is implement by an entry point."""
        if not name.startswith("do_"):
            raise AttributeError(name)
        data=list(pkg_resources.iter_entry_points("zopectl.command", name=name[3:]))
        if not data:
            raise AttributeError(name)
        if len(data)>1:
            print >>sys.stderr, "Warning: multiple entry points found for command"
            return
        func=data[0].load()
        if not callable(func):
            print >>sys.stderr, "Error: %s is not a callable method" % name
            return

        return self.run_entrypoint(data[0])


    def run_entrypoint(self, entry_point):
        def go(arg):
            # If the command line was something like
            # """bin/instance run "one two" three"""
            # cmd.parseline will have converted it so
            # that arg == 'one two three'. This is going to
            # foul up any quoted command with embedded spaces.
            # So we have to return to self.options.args,
            # which is a tuple of command line args,
            # throwing away the "run" command at the beginning.
            #
            # Further complications: if self.options.args has come
            # via subprocess, it may look like
            # ['run "arg 1" "arg2"'] rather than ['run','arg 1','arg2'].
            # If that's the case, we'll use csv to do the parsing
            # so that we can split on spaces while respecting quotes.
            tup = self.options.args
            if len(tup) == 1:
                tup = csv.reader(tup, delimiter=' ').next()

            # Remove -c and add command name as sys.argv[0]
            cmd = [ 'import sys',
                    'sys.argv.pop()',
                    'sys.argv.append(r\'%s\')' % entry_point.name
                   ]
            if len(tup) > 1:
                argv = tup[1:]
                for a in argv:
                    cmd.append('sys.argv.append(r\'%s\')' % a)
            cmd.extend([
                'import pkg_resources',
                'import Zope2',
                'func=pkg_resources.EntryPoint.parse(\'%s\').load(False)' % entry_point,
                'app=Zope2.app()',
                'func(app, sys.argv[1:])',
                ])
            cmdline = self.get_startup_cmd(self.options.python, ' ; '.join(cmd))
            self._exitstatus = os.system(cmdline)
        return go


    def do_run(self, args):
        if not args:
            print "usage: run <script> [args]"
            return
        # replace sys.argv
        script = args.split(' ')[0]
        cmd = (
            "import sys; sys.argv[:]=%r.split(' ');"
            "import Zope2; app=Zope2.app(); execfile(%r)"
            ) % (args,script)
        cmdline = self.get_startup_cmd(self.options.python, cmd)
        self._exitstatus = os.system(cmdline)

    def help_run(self):
        print "run <script> [args] -- run a Python script with the Zope "
        print "                       environment set up.  The script can use "
        print "                       the name 'app' access the top-level Zope"
        print "                       object"

    def do_adduser(self, arg):
        try:
            name, password = arg.split()
        except:
            print "usage: adduser <name> <password>"
            return
        cmdline = self.get_startup_cmd(
            self.options.python ,
            'import Zope2; '
            'app = Zope2.app(); '
            'result = app.acl_users._doAddUser(\'%s\', \'%s\', [\'Manager\'], []); '
            'import transaction; '
            'transaction.commit(); '
            'print \'Created user:\', result'
            ) % (name, password)
        os.system(cmdline)

    def help_adduser(self):
        print "adduser <name> <password> -- add a Zope management user"


def main(args=None):
    # This is exactly like zdctl.main(), but uses ZopeCtlOptions and
    # ZopeCmd instead of ZDCtlOptions and ZDCmd, so the default values
    # are handled properly for Zope.
    options = ZopeCtlOptions()
    options.realize(args)
    c = ZopeCmd(options)
    if options.args:
        c.onecmd(" ".join(options.args))
    else:
        options.interactive = 1
    if options.interactive:
        try:
            import readline
        except ImportError:
            pass
        print "program:", " ".join(options.program)
        c.do_status()
        c.cmdloop()
    else:
        return min(c._exitstatus, 1)

def _ignoreSIGCHLD(*unused):
    while 1:
        try: os.waitpid(-1, os.WNOHANG)
        except OSError: break

def run():
    # we don't care to be notified of our childrens' exit statuses.
    # this prevents zombie processes from cluttering up the process
    # table when zopectl start/stop is used interactively.
    # DM 2004-11-26: from the Linux "execve(2)" manual page:
    #     Any signals set to be caught by the calling process are reset
    #     to their default behaviour.
    #     The SIGCHLD signal (when set to SIG_IGN) may or may not be reset
    #     to SIG_DFL. 
    #   If it is not reset, 'os.wait[pid]' can non-deterministically fail.
    #   Thus, use a way such that "SIGCHLD" is definitely reset in children.
    #signal.signal(signal.SIGCHLD, signal.SIG_IGN)
    if not WIN and os.uname()[0] != 'Darwin':
        # On Windows the os.uname method does not exist.
        # On Mac OS X, setting up a signal handler causes waitpid to
        # raise EINTR, which is not preventable via the Python signal
        # handler API and can't be dealt with properly as we can't pass
        # the SA_RESTART to the signal API. Since Mac OS X doesn't
        # appear to clutter up the process table with zombies if
        # SIGCHILD is unset, just don't bother registering a SIGCHILD
        # signal handler at all.
        signal.signal(signal.SIGCHLD, _ignoreSIGCHLD)
    exitstatus = main()
    sys.exit(exitstatus)

if __name__ == '__main__':
    run()



"""SiteRoot module

Defines the Traverser base class and SiteRoot class
"""
from cgi import escape
import os

from Acquisition import Implicit
from App.Dialogs import MessageDialog
from App.special_dtml import DTMLFile
from OFS.SimpleItem import Item
from Persistence import Persistent
from ZPublisher.BeforeTraverse import NameCaller
from ZPublisher.BeforeTraverse import registerBeforeTraverse
from ZPublisher.BeforeTraverse import unregisterBeforeTraverse

SUPPRESS_SITEROOT = os.environ.has_key('SUPPRESS_SITEROOT')

class Traverser(Persistent, Item):
    """ Class for overriding container's __before_traverse__

    Containers are expected to have at most one instance of any particular
    subclass, with Id equal to the meta_type of the subclass.
    """
    meta_type = 'Traverser'
    priority = 100

    __ac_permissions__=()

    def addToContainer(self, container):
        container._setObject(self.id, self)
        self.manage_afterAdd(self, container)

    def manage_addToContainer(self, container, nextURL=''):
        if nextURL:
            if hasattr(getattr(container, 'aq_base', container), self.id):
                return MessageDialog(title='Item Exists',
                  message='This object already contains a %s' % self.meta_type,
                  action=nextURL)
        self.addToContainer(container)
        if nextURL:
            return MessageDialog(title='Item Added',
              message='This object now has a %s' % escape(self.meta_type),
              action=nextURL)

    def manage_beforeDelete(self, item, container):
        if item is self:
            unregisterBeforeTraverse(container, self.meta_type)

    def manage_afterAdd(self, item, container):
        if item is self:
            id = self.id
            if callable(id): id = id()

            # We want the original object, not stuff in between
            container = container.this()
            hook = NameCaller(id)
            registerBeforeTraverse(container, hook,
                                                  self.meta_type,
                                                  self.priority)
    def _setId(self, id):
        if id != self.id:
            raise ValueError('Cannot change the id of a %s'
                                % escape(self.meta_type))

class SiteRoot(Traverser, Implicit):
    """ SiteAccess.SiteRoot object

    A SiteRoot causes traversal of its container to replace the part
    of the Request path traversed so far with the request's SiteRootURL.
    """
    id = meta_type = 'SiteRoot'
    title = ''
    priority = 50

    manage_options=({'label':'Edit',
                     'action':'manage_main',
                     'help': ('SiteAccess', 'SiteRoot_Edit.stx'),
                    },)

    manage = manage_main = DTMLFile('www/SiteRootEdit', globals())
    manage_main._setName('manage_main')

    def __init__(self, title, base, path):
        self.title = title.strip()
        self.base = base = base.strip()
        self.path = path = path.strip()

    def manage_edit(self, title, base, path, REQUEST=None):
        """ Set the title, base, and path.
        """
        self.__init__(title, base, path)
        if REQUEST:
            return MessageDialog(title='SiteRoot changed.',
              message='SiteRoot changed.',
              action='%s/manage_main' % REQUEST['URL1'])

    def __call__(self, client, request, response=None):
        """ Traversing.
        """
        rq = request
        if SUPPRESS_SITEROOT:
            return
        base = (self.base or
                rq.get('SiteRootBASE') or
                rq.environ.get('SiteRootBASE'))
        path = (self.path or
                rq.get('SiteRootPATH') or
                rq.environ.get('SiteRootPATH'))
        if base is not None:
            rq['ACTUAL_URL'] = rq['ACTUAL_URL'].replace(rq['SERVER_URL'], base)
            rq['SERVER_URL'] = base
            rq._resetURLS()
        if path is not None:
            old = rq['URL']
            rq.setVirtualRoot(path)
            rq['ACTUAL_URL'] = rq['ACTUAL_URL'].replace(old, rq['URL'])

    def get_size(self):
        """ Make FTP happy
        """
        return 0

def manage_addSiteRoot(self, title='', base='', path='', REQUEST=None,
                       **ignored):
    """ Add a SiteRoot to a container.
    """
    sr = SiteRoot(title, base, path)
    if REQUEST:
        return sr.manage_addToContainer(self.this(),
                                        '%s/manage_main' % REQUEST['URL1'])
    else:
        sr.manage_addToContainer(self.this())

constructors = (
  ('manage_addSiteRootForm', DTMLFile('www/SiteRootAdd', globals())),
  ('manage_addSiteRoot', manage_addSiteRoot),
)



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""TaintedString legacy module.
"""

from zope.deferredimport import deprecated
deprecated('ZPublisher.TaintedString will be removed in Zope 2.14. Please '
           'import from AccessControl.tainted instead.',
           TaintedString = 'AccessControl.tainted:TaintedString',
          )



##############################################################################
#
# Copyright (c) 2004, 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Five security handling
"""

from zope.deferredimport import deprecated

deprecated("Please import InitializeClass from App.class_init",
    initializeClass = 'App.class_init:InitializeClass',
)

deprecated("Please import from AccessControl.security",
    CheckerPublicId = 'AccessControl.security:CheckerPublicId',
    CheckerPrivateId = 'AccessControl.security:CheckerPrivateId',
    getSecurityInfo = 'AccessControl.security:getSecurityInfo',
    clearSecurityInfo = 'AccessControl.security:clearSecurityInfo',
    checkPermission = 'AccessControl.security:checkPermission',
    FiveSecurityPolicy = 'AccessControl.security:SecurityPolicy',
    newInteraction = 'AccessControl.security:newInteraction',
    _getSecurity = 'AccessControl.security:_getSecurity',
    protectName = 'AccessControl.security:protectName',
    protectClass = 'AccessControl.security:protectClass',
)



import unittest

class ConfigTestBase:

    def setUp(self):
        import App.config
        self._old_config = App.config._config

    def tearDown(self):
        import App.config
        App.config._config = self._old_config

    def _makeConfig(self, **kw):
        import App.config
        class DummyConfig:
            pass
        App.config._config = config = DummyConfig()
        config.dbtab = DummyDBTab(kw)
        return config

class FakeConnectionTests(unittest.TestCase):

    def _getTargetClass(self):
        from App.ApplicationManager import FakeConnection
        return FakeConnection

    def _makeOne(self, db, parent_jar):
        return self._getTargetClass()(db, parent_jar)

    def test_holds_db(self):
        db = object()
        parent_jar = object()
        fc = self._makeOne(db, parent_jar)
        self.assertTrue(fc.db() is db)

class DatabaseChooserTests(ConfigTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from App.ApplicationManager import DatabaseChooser
        return DatabaseChooser

    def _makeOne(self, id):
        return self._getTargetClass()(id)

    def _makeRoot(self):
        from ExtensionClass import Base
        class Root(Base):
            _p_jar = None
            def getPhysicalRoot(self):
                return self
        return Root()

    def test_getDatabaseNames_sorted(self):
        self._makeConfig(foo=object(), bar=object(), qux=object())
        dc = self._makeOne('test')
        self.assertEqual(list(dc.getDatabaseNames()), ['bar', 'foo', 'qux'])

    def test___getitem___miss(self):
        self._makeConfig(foo=object(), bar=object(), qux=object())
        dc = self._makeOne('test')
        self.assertRaises(KeyError, dc.__getitem__, 'nonesuch')

    def test___getitem___hit(self):
        from App.ApplicationManager import AltDatabaseManager
        from App.ApplicationManager import FakeConnection
        foo=object()
        bar=object()
        qux=object()
        self._makeConfig(foo=foo, bar=bar, qux=qux)
        root = self._makeRoot()
        dc = self._makeOne('test').__of__(root)
        found = dc['foo']
        self.assertTrue(isinstance(found, AltDatabaseManager))
        self.assertEqual(found.id, 'foo')
        self.assertTrue(found.aq_parent is dc)
        conn = found._p_jar
        self.assertTrue(isinstance(conn, FakeConnection))
        self.assertTrue(conn.db() is foo)

    def test___bobo_traverse___miss(self):
        self._makeConfig(foo=object(), bar=object(), qux=object())
        dc = self._makeOne('test')
        self.assertRaises(AttributeError,
                          dc.__bobo_traverse__, None, 'nonesuch')

    def test___bobo_traverse___hit_db(self):
        from App.ApplicationManager import AltDatabaseManager
        from App.ApplicationManager import FakeConnection
        foo=object()
        bar=object()
        qux=object()
        self._makeConfig(foo=foo, bar=bar, qux=qux)
        root = self._makeRoot()
        dc = self._makeOne('test').__of__(root)
        found = dc.__bobo_traverse__(None, 'foo')
        self.assertTrue(isinstance(found, AltDatabaseManager))
        self.assertEqual(found.id, 'foo')
        self.assertTrue(found.aq_parent is dc)
        conn = found._p_jar
        self.assertTrue(isinstance(conn, FakeConnection))
        self.assertTrue(conn.db() is foo)

    def test___bobo_traverse___miss_db_hit_attr(self):
        foo=object()
        bar=object()
        qux=object()
        self._makeConfig(foo=foo, bar=bar, qux=qux)
        root = self._makeRoot()
        dc = self._makeOne('test').__of__(root)
        dc.spam = spam = object()
        found = dc.__bobo_traverse__(None, 'spam')
        self.assertTrue(found is spam)

    def test_tpValues(self):
        from App.ApplicationManager import AltDatabaseManager
        foo=object()
        bar=object()
        qux=object()
        self._makeConfig(foo=foo, bar=bar, qux=qux)
        root = self._makeRoot()
        dc = self._makeOne('test').__of__(root)
        values = dc.tpValues()
        self.assertEqual(len(values), 3)
        self.assertTrue(isinstance(values[0], AltDatabaseManager))
        self.assertEqual(values[0].id, 'bar')
        self.assertEqual(values[0]._p_jar, None)
        self.assertTrue(isinstance(values[1], AltDatabaseManager))
        self.assertEqual(values[1].id, 'foo')
        self.assertEqual(values[1]._p_jar, None)
        self.assertTrue(isinstance(values[2], AltDatabaseManager))
        self.assertEqual(values[2].id, 'qux')
        self.assertEqual(values[2]._p_jar, None)


class DebugManagerTests(unittest.TestCase):

    def setUp(self):
        import sys
        self._sys = sys
        self._old_sys_modules = sys.modules.copy()

    def tearDown(self):
        self._sys.modules.clear()
        self._sys.modules.update(self._old_sys_modules)

    def _getTargetClass(self):
        from App.ApplicationManager import DebugManager
        return DebugManager

    def _makeOne(self, id):
        return self._getTargetClass()(id)

    def _makeModuleClasses(self):
        import sys
        import types
        from ExtensionClass import Base
        class Foo(Base):
            pass
        class Bar(Base):
            pass
        class Baz(Base):
            pass
        foo = sys.modules['foo'] = types.ModuleType('foo')
        foo.Foo = Foo
        Foo.__module__ = 'foo'
        foo.Bar = Bar
        Bar.__module__ = 'foo'
        qux = sys.modules['qux'] = types.ModuleType('qux')
        qux.Baz = Baz
        Baz.__module__ = 'qux'
        return Foo, Bar, Baz

    def test_refcount_no_limit(self):
        import sys
        dm = self._makeOne('test')
        Foo, Bar, Baz = self._makeModuleClasses()
        pairs = dm.refcount()
        # XXX : Ugly empiricism here:  I don't know why the count is up 1.
        foo_count = sys.getrefcount(Foo)
        self.assertTrue((foo_count+1, 'foo.Foo') in pairs)
        bar_count = sys.getrefcount(Bar)
        self.assertTrue((bar_count+1, 'foo.Bar') in pairs)
        baz_count = sys.getrefcount(Baz)
        self.assertTrue((baz_count+1, 'qux.Baz') in pairs)

    def test_refdict(self):
        import sys
        dm = self._makeOne('test')
        Foo, Bar, Baz = self._makeModuleClasses()
        mapping = dm.refdict()
        # XXX : Ugly empiricism here:  I don't know why the count is up 1.
        foo_count = sys.getrefcount(Foo)
        self.assertEqual(mapping['foo.Foo'], foo_count+1)
        bar_count = sys.getrefcount(Bar)
        self.assertEqual(mapping['foo.Bar'], bar_count+1)
        baz_count = sys.getrefcount(Baz)
        self.assertEqual(mapping['qux.Baz'], baz_count+1)

    def test_rcsnapshot(self):
        import sys
        import App.ApplicationManager
        from DateTime.DateTime import DateTime
        dm = self._makeOne('test')
        Foo, Bar, Baz = self._makeModuleClasses()
        before = DateTime()
        dm.rcsnapshot()
        after = DateTime()
        # XXX : Ugly empiricism here:  I don't know why the count is up 1.
        self.assertTrue(before <= App.ApplicationManager._v_rst <= after)
        mapping = App.ApplicationManager._v_rcs
        foo_count = sys.getrefcount(Foo)
        self.assertEqual(mapping['foo.Foo'], foo_count+1)
        bar_count = sys.getrefcount(Bar)
        self.assertEqual(mapping['foo.Bar'], bar_count+1)
        baz_count = sys.getrefcount(Baz)
        self.assertEqual(mapping['qux.Baz'], baz_count+1)

    def test_rcdate(self):
        import App.ApplicationManager
        dummy = object()
        App.ApplicationManager._v_rst = dummy
        dm = self._makeOne('test')
        found = dm.rcdate()
        App.ApplicationManager._v_rst = None
        self.assertTrue(found is dummy)

    def test_rcdeltas(self):
        dm = self._makeOne('test')
        dm.rcsnapshot()
        Foo, Bar, Baz = self._makeModuleClasses()
        mappings = dm.rcdeltas()
        self.assertTrue(len(mappings))
        mapping = mappings[0]
        self.assertTrue('rc' in mapping)
        self.assertTrue('pc' in mapping)
        self.assertEqual(mapping['delta'], mapping['rc'] - mapping['pc'])

    #def test_dbconnections(self):  XXX -- TOO UGLY TO TEST
    #def test_manage_profile_stats(self):  XXX -- TOO UGLY TO TEST

    def test_manage_profile_reset(self):
        import sys
        from ZPublisher import Publish
        _old_sys__ps_ = getattr(sys, '_ps_', self)
        _old_Publish_pstat = getattr(Publish, '_pstat', self)
        sys._ps_ = Publish._pstat = object()
        try:
            dm = self._makeOne('test')
            dm.manage_profile_reset()
        finally:
            if _old_sys__ps_ is not self:
                sys._ps_ = _old_sys__ps_
            if _old_Publish_pstat is not self:
                Publish._pstat = _old_Publish_pstat
        self.assertTrue(sys._ps_ is None)
        self.assertTrue(Publish._pstat is None)

    def test_manage_getSysPath(self):
        import sys
        dm = self._makeOne('test')
        self.assertEqual(dm.manage_getSysPath(), list(sys.path))


class DBProxyTestsBase:

    def _makeOne(self):
        return self._getTargetClass()()

    def _makeJar(self, dbname, dbsize):
        class Jar:
            def db(self):
                return self._db
        jar = Jar()
        jar._db = DummyDB(dbname, dbsize)
        return jar

    def test_db_name(self):
        am = self._makeOne()
        am._p_jar = self._makeJar('foo', '')
        self.assertEqual(am.db_name(), 'foo')

    def test_db_size_string(self):
        am = self._makeOne()
        am._p_jar = self._makeJar('foo', 'super')
        self.assertEqual(am.db_size(), 'super')

    def test_db_size_lt_1_meg(self):
        am = self._makeOne()
        am._p_jar = self._makeJar('foo', 4497)
        self.assertEqual(am.db_size(), '4.4K')

    def test_db_size_gt_1_meg(self):
        am = self._makeOne()
        am._p_jar = self._makeJar('foo', (2048 * 1024) + 123240)
        self.assertEqual(am.db_size(), '2.1M')

    def test_manage_pack(self):
        am = self._makeOne()
        jar = am._p_jar = self._makeJar('foo', '')
        am.manage_pack(1, _when=86400*2)
        self.assertEqual(jar._db._packed, 86400)

class ApplicationManagerTests(ConfigTestBase,
                              DBProxyTestsBase,
                              unittest.TestCase,
                             ):

    def setUp(self):
        ConfigTestBase.setUp(self)
        self._tempdirs = ()

    def tearDown(self):
        import shutil
        for tempdir in self._tempdirs:
            shutil.rmtree(tempdir)
        ConfigTestBase.tearDown(self)

    def _getTargetClass(self):
        from App.ApplicationManager import ApplicationManager
        return ApplicationManager

    def _makeTempdir(self):
        import tempfile
        tmp = tempfile.mkdtemp()
        self._tempdirs += (tmp,)
        return tmp

    def _makeFile(self, dir, name, text):
        import os
        os.makedirs(dir)
        fqn = os.path.join(dir, name)
        f = open(fqn, 'w')
        f.write(text)
        f.flush()
        f.close()
        return fqn

    def test_version_txt(self):
        from App.version_txt import version_txt
        am = self._makeOne()
        self.assertEqual(am.version_txt(), version_txt())

    def test_sys_version(self):
        import sys
        am = self._makeOne()
        self.assertEqual(am.sys_version(), sys.version)

    def test_sys_platform(self):
        import sys
        am = self._makeOne()
        self.assertEqual(am.sys_platform(), sys.platform)

    def test_ctor_initializes_Products(self):
        from App.Product import ProductFolder
        am = self._makeOne()
        self.assertTrue(isinstance(am.Products, ProductFolder))

    def test__canCopy(self):
        am = self._makeOne()
        self.assertFalse(am._canCopy())

    def test_manage_app(self):
        from zExceptions import Redirect
        am = self._makeOne()
        try:
            am.manage_app('http://example.com/foo')
        except Redirect, v:
            self.assertEqual(v.args, ('http://example.com/foo/manage',))
        else:
            self.fail('Redirect not raised')

    def test_process_time_seconds(self):
        am = self._makeOne()
        am.process_start = 0
        self.assertEqual(am.process_time(0).strip(), '0 sec')
        self.assertEqual(am.process_time(1).strip(), '1 sec')
        self.assertEqual(am.process_time(2).strip(), '2 sec')

    def test_process_time_minutes(self):
        am = self._makeOne()
        am.process_start = 0
        self.assertEqual(am.process_time(60).strip(), '1 min 0 sec')
        self.assertEqual(am.process_time(61).strip(), '1 min 1 sec')
        self.assertEqual(am.process_time(62).strip(), '1 min 2 sec')
        self.assertEqual(am.process_time(120).strip(), '2 min 0 sec')
        self.assertEqual(am.process_time(121).strip(), '2 min 1 sec')
        self.assertEqual(am.process_time(122).strip(), '2 min 2 sec')

    def test_process_time_hours(self):
        am = self._makeOne()
        am.process_start = 0
        n1 = 60 * 60
        n2 = n1 * 2
        self.assertEqual(am.process_time(n1).strip(),
                         '1 hour  0 sec')
        self.assertEqual(am.process_time(n1 + 61).strip(),
                         '1 hour 1 min 1 sec')
        self.assertEqual(am.process_time(n2 + 1).strip(),
                         '2 hours  1 sec')
        self.assertEqual(am.process_time(n2 + 122).strip(),
                         '2 hours 2 min 2 sec')

    def test_process_time_days(self):
        am = self._makeOne()
        am.process_start = 0
        n1 = 60 * 60 * 24
        n2 = n1 * 2
        self.assertEqual(am.process_time(n1).strip(),
                         '1 day   0 sec')
        self.assertEqual(am.process_time(n1 + 3661).strip(),
                         '1 day 1 hour 1 min 1 sec')
        self.assertEqual(am.process_time(n2 + 1).strip(),
                         '2 days   1 sec')
        self.assertEqual(am.process_time(n2 + 7322).strip(),
                         '2 days 2 hours 2 min 2 sec')

    def test_thread_get_ident(self):
        import thread
        am = self._makeOne()
        self.assertEqual(am.thread_get_ident(), thread.get_ident())

    #def test_manage_restart(self):  XXX -- TOO UGLY TO TEST
    #def test_manage_restart(self):  XXX -- TOO UGLY TO TEST

    def test_revert_points(self):
        am = self._makeOne()
        self.assertEqual(list(am.revert_points()), [])

    def test_version_list(self):
        # XXX this method is too stupid to live:  returning a bare list
        #     of versions without even tying them to the products?
        #     and what about products living outside SOFTWARE_HOME?
        #     Nobody calls it, either
        import os
        am = self._makeOne()
        config = self._makeConfig()
        swdir = config.softwarehome = self._makeTempdir()
        foodir = os.path.join(swdir, 'Products', 'foo')
        self._makeFile(foodir, 'VERSION.TXT', '1.2')
        bardir = os.path.join(swdir, 'Products', 'bar')
        self._makeFile(bardir, 'VERSION.txt', '3.4')
        bazdir = os.path.join(swdir, 'Products', 'baz')
        self._makeFile(bazdir, 'version.txt', '5.6')
        versions = am.version_list()
        self.assertEqual(versions, ['3.4', '5.6', '1.2'])

    def test_getSOFTWARE_HOME_missing(self):
        am = self._makeOne()
        config = self._makeConfig()
        self.assertEqual(am.getSOFTWARE_HOME(), None)

    def test_getSOFTWARE_HOME_present(self):
        am = self._makeOne()
        config = self._makeConfig()
        swdir = config.softwarehome = self._makeTempdir()
        self.assertEqual(am.getSOFTWARE_HOME(), swdir)

    def test_getZOPE_HOME_missing(self):
        am = self._makeOne()
        config = self._makeConfig()
        self.assertEqual(am.getZOPE_HOME(), None)

    def test_getZOPE_HOME_present(self):
        am = self._makeOne()
        config = self._makeConfig()
        zopedir = config.zopehome = self._makeTempdir()
        self.assertEqual(am.getZOPE_HOME(), zopedir)

    def test_getINSTANCE_HOME(self):
        am = self._makeOne()
        config = self._makeConfig()
        instdir = config.instancehome = self._makeTempdir()
        self.assertEqual(am.getINSTANCE_HOME(), instdir)

    def test_getCLIENT_HOME(self):
        am = self._makeOne()
        config = self._makeConfig()
        cldir = config.clienthome = self._makeTempdir()
        self.assertEqual(am.getCLIENT_HOME(), cldir)

    def test_getServers(self):
        from asyncore import socket_map

        class DummySocketServer:
            def __init__(self, port):
                self.port = port

        class AnotherSocketServer(DummySocketServer):
            pass

        class NotAServer:
            pass

        am = self._makeOne()
        _old_socket_map = socket_map.copy()
        socket_map.clear()
        socket_map['foo'] = DummySocketServer(45)
        socket_map['bar'] = AnotherSocketServer(57)
        socket_map['qux'] = NotAServer()

        try:
            pairs = am.getServers()
        finally:
            socket_map.clear()
            socket_map.update(_old_socket_map)

        self.assertEqual(len(pairs), 2)
        self.assertTrue((str(DummySocketServer), 'Port: 45') in pairs)
        self.assertTrue((str(AnotherSocketServer), 'Port: 57') in pairs)

    #def test_objectIds(self):  XXX -- TOO UGLY TO TEST (BBB for Zope 2.3!!)


class AltDatabaseManagerTests(DBProxyTestsBase,
                              unittest.TestCase,
                             ):

    def _getTargetClass(self):
        from App.ApplicationManager import AltDatabaseManager
        return AltDatabaseManager


class DummyDBTab:
    def __init__(self, databases=None):
        self._databases = databases or {}

    def listDatabaseNames(self):
        return self._databases.keys()

    def hasDatabase(self, name):
        return name in self._databases

    def getDatabase(self, name):
        return self._databases[name]

class DummyDB:

    _packed = None

    def __init__(self, name, size):
        self._name = name
        self._size = size

    def getName(self):
        return self._name

    def getSize(self):
        return self._size

    def pack(self, when):
        self._packed = when

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(FakeConnectionTests),
        unittest.makeSuite(DatabaseChooserTests),
        unittest.makeSuite(DebugManagerTests),
        unittest.makeSuite(ApplicationManagerTests),
        unittest.makeSuite(AltDatabaseManagerTests),
    ))



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import time

from Testing import makerequest
import ZODB # in order to get Persistence.Persistent working
import transaction
import Acquisition
from Products.Sessions.BrowserIdManager import BrowserIdManager, \
     getNewBrowserId
from Products.Sessions.SessionDataManager import \
    SessionDataManager
from Products.Transience.Transience import \
    TransientObjectContainer
from Products.TemporaryFolder.TemporaryFolder import MountedTemporaryFolder
from ZODB.POSException import ConflictError, \
     ReadConflictError, BTreesConflictError
from unittest import TestCase, TestSuite, makeSuite
import threading, random
from ZODB.DemoStorage import DemoStorage
from OFS.Application import Application
import traceback

from Products.Transience.tests import fauxtime
import Products.Transience.Transience
import Products.Transience.TransientObject
Products.Transience.Transience.time = fauxtime
Products.Transience.TransientObject.time = fauxtime

tf_name = 'temp_folder'
idmgr_name = 'browser_id_manager'
toc_name = 'temp_transient_container'
sdm_name = 'session_data_manager'

stuff = {}

def log_time():
    """Return a simple time string without spaces suitable for logging."""
    return ("%4.4d-%2.2d-%2.2dT%2.2d:%2.2d:%2.2d"
            % time.localtime()[:6])


def _getDB():
    db = stuff.get('db')
    if not db:
        ds = DemoStorage()
        db = ZODB.DB(ds)
        conn = db.open()
        root = conn.root()
        app = Application()
        root['Application']= app
        _populate(app)
        transaction.commit()
        stuff['db'] = db
        conn.close()
    return db

def _delDB():
    transaction.abort()
    del stuff['db']

class Foo(Acquisition.Implicit): pass

def _populate(app):
    bidmgr = BrowserIdManager(idmgr_name)
    tf = MountedTemporaryFolder(tf_name, 'Temporary Folder')
    toc = TransientObjectContainer(toc_name, title='Temporary '
        'Transient Object Container', timeout_mins=1)
    session_data_manager=SessionDataManager(id=sdm_name,
        path='/'+tf_name+'/'+toc_name, title='Session Data Manager')

    try: app._delObject(idmgr_name)
    except AttributeError: pass

    try: app._delObject(tf_name)
    except AttributeError: pass

    try: app._delObject(sdm_name)
    except AttributeError: pass

    app._setObject(idmgr_name, bidmgr)

    app._setObject(sdm_name, session_data_manager)

    app._setObject(tf_name, tf)
    transaction.commit()

    app.temp_folder._setObject(toc_name, toc)
    transaction.commit()

class TestMultiThread(TestCase):
    def testOverlappingBrowserIds(self):
        token = getNewBrowserId()
        self.go(token)

    def testNonOverlappingBrowserIds(self):
        token = None
        self.go(token)

    def go(self, token):
        readers = []
        writers = []
        valuers = []
        readiters = 3
        writeiters = 3
        valueiters = 3
        numreaders = 2
        numwriters = 4
        numvaluers = 1
        db = _getDB()
        for i in range(numreaders):
            thread = ReaderThread(db, readiters, token)
            readers.append(thread)
        for i in range(numvaluers):
            thread = ValuesGetterThread(db, valueiters, token)
            valuers.append(thread)
        for i in range(numwriters):
            thread = WriterThread(db, writeiters, token)
            writers.append(thread)
        for thread in readers:
            thread.start()
            time.sleep(0.1)
        for thread in writers:
            thread.start()
            time.sleep(0.1)
        for thread in valuers:
            thread.start()
            time.sleep(0.1)
        active = threading.activeCount()
        while active > 0:
            active = threading.activeCount()-1
            print 'waiting for %s threads' % active
            print "readers: ", numActive(readers),
            print "writers: ", numActive(writers),
            print "valuers: ", numActive(valuers)
            time.sleep(5)

def numActive(threads):
    i = 0
    for thread in threads:
        if not thread.isFinished():
            i+=1
    return i

class BaseReaderWriter(threading.Thread):
    def __init__(self, db, iters, token=None):
        self.iters = iters
        self.sdm_name = sdm_name
        self.finished = 0
        self.db = db
        self.token = token
        threading.Thread.__init__(self)

    def run(self):
        i = 0
        try:
            while 1:
                self.conn = self.db.open()
                self.app = self.conn.root()['Application']
                self.app = makerequest.makerequest(self.app)
                if self.token is None:
                    token = getNewBrowserId()
                else:
                    token = self.token
                    self.app.REQUEST.browser_id_ = token

                try:
                    self.run1()
                    return
                except ReadConflictError:
                    #traceback.print_exc()
                    print "R",
                except BTreesConflictError:
                    print "B",
                except ConflictError:
                    print "W",
                except:
                    transaction.abort()
                    print log_time()
                    traceback.print_exc()
                    raise
                
                i = i + 1
                transaction.abort()
                self.conn.close()
                time.sleep(random.randrange(10) * .1)
        finally:
            transaction.abort()
            self.conn.close()
            del self.app
            self.finished = 1
            print '%s finished' % self.__class__

    def isFinished(self):
        return self.finished
        

class ReaderThread(BaseReaderWriter):
    def run1(self):
        session_data_manager = getattr(self.app, self.sdm_name)
        data = session_data_manager.getSessionData(create=1)
        t = time.time()
        data[t] = 1
        transaction.commit()
        for i in range(self.iters):
            data = session_data_manager.getSessionData()
            time.sleep(random.choice(range(3)))
            transaction.commit()

class WriterThread(BaseReaderWriter):
    def run1(self):
        session_data_manager = getattr(self.app, self.sdm_name)
        for i in range(self.iters):
            data = session_data_manager.getSessionData()
            data[time.time()] = 1
            n = random.choice(range(3))
            time.sleep(n)
            if n % 2 == 0:
                transaction.commit()
            else:
                transaction.abort()

class ValuesGetterThread(BaseReaderWriter):
    def run1(self):
        tf = getattr(self.app, tf_name)
        toc = getattr(tf, toc_name)
        for i in range(self.iters):
            print '%s values in toc' % len(toc.values())
            n = random.choice(range(3))
            time.sleep(n)
            if n % 2 == 0:
                transaction.commit()
            else:
                transaction.abort()


def test_suite():
    test_multithread = makeSuite(TestMultiThread, 'test')
    suite = TestSuite((test_multithread,))
    return suite



"""SiteRoot regression tests.

These tests verify that the request URL headers, in particular ACTUAL_URL, are
set correctly when a SiteRoot is used.

See http://www.zope.org/Collectors/Zope/2077
"""
import unittest


class TraverserTests(unittest.TestCase):

    def _getTargetClass(self):
        from Products.SiteAccess.SiteRoot import Traverser
        return Traverser

    def _makeOne(self):
        traverser = self._getTargetClass()()
        traverser.id = 'testing'
        return traverser

    def test_addToContainer(self):
        traverser = self._makeOne()
        container = DummyContainer()
        traverser.addToContainer(container)
        self.assertTrue(container.testing is traverser)
        hook = container.__before_traverse__[(100, 'Traverser')]
        self.assertEqual(hook.name, 'testing')

    def test_manage_addToContainer_no_nextUrl(self):
        traverser = self._makeOne()
        container = DummyContainer()
        result = traverser.manage_addToContainer(container)
        self.assertTrue(result is None)
        self.assertTrue(container.testing is traverser)
        hook = container.__before_traverse__[(100, 'Traverser')]
        self.assertEqual(hook.name, 'testing')

    def test_manage_addToContainer_w_nextUrl_w_name_collision(self):
        NEXTURL='http://example.com/manage_main'
        traverser = self._makeOne()
        container = DummyContainer()
        container.testing = object()
        result = traverser.manage_addToContainer(container, nextURL=NEXTURL)
        self.assertTrue('<TITLE>Item Exists</TITLE>' in result)
        self.assertFalse(container.testing is traverser)

    def test_manage_addToContainer_w_nextUrl_wo_name_collision(self):
        NEXTURL='http://example.com/manage_main'
        traverser = self._makeOne()
        container = DummyContainer()
        result = traverser.manage_addToContainer(container, nextURL=NEXTURL)
        self.assertTrue('<TITLE>Item Added</TITLE>' in result)
        self.assertTrue(container.testing is traverser)
        hook = container.__before_traverse__[(100, 'Traverser')]
        self.assertEqual(hook.name, 'testing')

    def test_manage_beforeDelete_item_is_not_self(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        traverser = self._makeOne()
        container = DummyContainer()
        other = container.other = DummyObject(name='other')
        registerBeforeTraverse(container, other, 'Traverser', 100)
        item = object()
        traverser.manage_beforeDelete(item, container)
        hook = container.__before_traverse__[(100, 'Traverser')]
        self.assertEqual(hook.name, 'other')

    def test_manage_beforeDelete_item_is_self(self):
        from ZPublisher.BeforeTraverse import registerBeforeTraverse
        traverser = self._makeOne()
        container = DummyContainer()
        other = container.other = DummyObject(name='other')
        registerBeforeTraverse(container, other, 'Traverser', 100)
        traverser.manage_beforeDelete(traverser, container)
        self.assertFalse(container.__before_traverse__)

    def test_manage_afterAdd_item_not_self(self):
        traverser = self._makeOne()
        container = DummyContainer()
        item = object()
        traverser.manage_afterAdd(item, container)
        self.assertFalse('__before_traverse__' in container.__dict__)

    def test_manage_afterAdd_item_is_self(self):
        traverser = self._makeOne()
        container = DummyContainer()
        traverser.manage_afterAdd(traverser, container)
        hook = container.__before_traverse__[(100, 'Traverser')]
        self.assertEqual(hook.name, 'testing')

    def test__setId_same(self):
        traverser = self._makeOne()
        traverser._setId('testing') # doesn't raise

    def test__setId_different(self):
        traverser = self._makeOne()
        self.assertRaises(ValueError, traverser._setId, 'other')


class SiteRootTests(unittest.TestCase):

    _old_SSR = None

    def setUp(self):
        from Testing.ZopeTestCase import ZopeLite
        ZopeLite.startup()

    def tearDown(self):
        if self._old_SSR is not None:
            self._set_SUPPRESS_SITEROOT(self._old_SSR)

    def _set_SUPPRESS_SITEROOT(self, value):
        from Products.SiteAccess import SiteRoot as SR
        (self._old_SSR,
         SR.SUPPRESS_SITEROOT) = (SR.SUPPRESS_SITEROOT, value)

    def _getTargetClass(self):
        from Products.SiteAccess.SiteRoot import SiteRoot
        return SiteRoot

    def _makeOne(self, title='TITLE', base='', path=''):
        return self._getTargetClass()(title, base, path)

    def test___init___strips_base_and_path(self):
        siteroot = self._makeOne(base=' ', path=' ')
        self.assertEqual(siteroot.title, 'TITLE')
        self.assertEqual(siteroot.base, '')
        self.assertEqual(siteroot.path, '')

    def test___init___w_base_and_path(self):
        siteroot = self._makeOne(base='http://example.com', path='/path')
        self.assertEqual(siteroot.title, 'TITLE')
        self.assertEqual(siteroot.base, 'http://example.com')
        self.assertEqual(siteroot.path, '/path')

    def test_manage_edit_no_REQUEST(self):
        siteroot = self._makeOne(title='Before',
                                 base='http://before.example.com',
                                 path='/before')
        result = siteroot.manage_edit('After', 'http://after.example.com ',
                                      '/after ')
        self.assertTrue(result is None)
        self.assertEqual(siteroot.title, 'After')
        self.assertEqual(siteroot.base, 'http://after.example.com')
        self.assertEqual(siteroot.path, '/after')

    def test_manage_edit_w_REQUEST(self):
        siteroot = self._makeOne(title='Before',
                                 base='http://before.example.com',
                                 path='/before')
        result = siteroot.manage_edit('After', 'http://after.example.com ',
                                      '/after ',
                                      REQUEST = {'URL1':
                                        'http://localhost:8080/manage_main'})
        self.assertTrue('<TITLE>SiteRoot changed.</TITLE>' in result)
        self.assertEqual(siteroot.title, 'After')
        self.assertEqual(siteroot.base, 'http://after.example.com')
        self.assertEqual(siteroot.path, '/after')

    def test___call___w_SUPPRESS_SITEROOT_set(self):
        self._set_SUPPRESS_SITEROOT(1)
        siteroot = self._makeOne(base='http://example.com', path='/path')
        request = {}
        siteroot(None, request)
        self.assertEqual(request, {})

    def test___call___w_SUPPRESS_SITEROOT_in_URL(self):
        # This behavior changed in landing lp:142878.
        URL='http://localhost:8080/example/folder/'
        siteroot = self._makeOne(base='http://example.com', path='/example')
        request = DummyRequest(TraversalRequestNameStack=
                                    ['_SUPPRESS_SITEROOT'],
                               URL=URL,
                               ACTUAL_URL=URL,
                               SERVER_URL='http://localhost:8080',
                              )
        request.steps = []
        request.environ = {}
        siteroot(None, request)
        self.assertEqual(request['URL'], URL)
        self.assertEqual(request['SERVER_URL'], 'http://example.com')
        self.assertEqual(request['ACTUAL_URL'], 
                         'http://example.com/example/folder/')
        self.assertEqual(request._virtual_root, '/example')
        self.assertTrue(request._urls_reset)

    def test___call___wo_SUPPRESS_SITEROOT_w_base_wo_path(self):
        URL='http://localhost:8080/example/folder/'
        siteroot = self._makeOne(base='http://example.com', path='')
        request = DummyRequest(TraversalRequestNameStack=[],
                               URL=URL,
                               ACTUAL_URL=URL,
                               SERVER_URL='http://localhost:8080',
                              )
        request.steps = []
        request.environ = {}
        siteroot(None, request)
        self.assertEqual(request['URL'], URL)
        self.assertEqual(request['SERVER_URL'], 'http://example.com')
        self.assertEqual(request['ACTUAL_URL'],
                         'http://example.com/example/folder/')
        self.assertEqual(request._virtual_root, None)
        self.assertTrue(request._urls_reset)

    def test___call___wo_SUPPRESS_SITEROOT_wo_base_w_path(self):
        URL='http://localhost:8080/example/folder/'
        siteroot = self._makeOne(base='', path='/example')
        request = DummyRequest(TraversalRequestNameStack=[],
                               URL=URL,
                               ACTUAL_URL=URL,
                               SERVER_URL='http://localhost:8080',
                              )
        request.steps = []
        request.environ = {}
        siteroot(None, request)
        self.assertEqual(request['URL'], URL)
        self.assertEqual(request['SERVER_URL'], 'http://localhost:8080')
        self.assertEqual(request['ACTUAL_URL'], URL)
        self.assertEqual(request._virtual_root, '/example')
        self.assertFalse(request._urls_reset)

    def test___call___wo_SUPPRESS_SITEROOT_w_base_w_path(self):
        URL='http://localhost:8080/example/folder/'
        siteroot = self._makeOne(base='http://example.com', path='/example')
        request = DummyRequest(TraversalRequestNameStack=[],
                               URL=URL,
                               ACTUAL_URL=URL,
                               SERVER_URL='http://localhost:8080',
                              )
        request.steps = []
        request.environ = {}
        siteroot(None, request)
        self.assertEqual(request['URL'], URL)
        self.assertEqual(request['SERVER_URL'], 'http://example.com')
        self.assertEqual(request['ACTUAL_URL'], 
                         'http://example.com/example/folder/')
        self.assertEqual(request._virtual_root, '/example')
        self.assertTrue(request._urls_reset)

    def test_get_size(self):
        siteroot = self._makeOne()
        self.assertEqual(siteroot.get_size(), 0)


class DummyObject(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)


class DummyContainer(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def _setObject(self, name, object):
        setattr(self, name, object)

    def this(self):
        return self


class DummyRequest(dict):

    _virtual_root = None
    _urls_reset = False

    def setVirtualRoot(self, root):
        self._virtual_root = root

    def _resetURLS(self):
        self._urls_reset = True


class SiteRootRegressions(unittest.TestCase):

    def setUp(self):
        import transaction
        from Testing.makerequest import makerequest
        from Testing.ZopeTestCase.ZopeLite import app
        transaction.begin()
        self.app = makerequest(app())
        self.app.manage_addFolder('folder')
        p_disp = self.app.folder.manage_addProduct['SiteAccess']
        p_disp.manage_addSiteRoot(title='SiteRoot',
                                    base='http://test_base',
                                    path='/test_path')
        self.app.REQUEST.set('PARENTS', [self.app])
        self.app.REQUEST.traverse('/folder')

    def tearDown(self):
        import transaction
        transaction.abort()
        self.app._p_jar.close()
        
    def testRequest(self):
        self.assertEqual(self.app.REQUEST['SERVER_URL'], 'http://test_base') 
        self.assertEqual(self.app.REQUEST['URL'],
                         'http://test_base/test_path/index_html')
        self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                         'http://test_base/test_path')
    def testAbsoluteUrl(self):            
        self.assertEqual(self.app.folder.absolute_url(),
                         'http://test_base/test_path')

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(TraverserTests),
        unittest.makeSuite(SiteRootTests),
        unittest.makeSuite(SiteRootRegressions),
    ))



import unittest
import os.path
import App
from Testing.ZopeTestCase.warnhook import WarningsHook


class TestImageFile(unittest.TestCase):

    def setUp(self):
        # ugly: need to save the old App.config configuration value since
        # ImageFile might read it and trigger setting it to the default value 
        self.oldcfg = App.config._config
        self.warningshook = WarningsHook()
        self.warningshook.install()

    def tearDown(self):
        self.warningshook.uninstall()
        # ugly: need to restore configuration, or lack thereof
        App.config._config = self.oldcfg

    def test_warn_on_software_home_default(self):
        App.ImageFile.ImageFile('App/www/zopelogo.jpg')
        self.assertEquals(self.warningshook.warnings.pop()[0],
                          App.ImageFile.NON_PREFIX_WARNING)

    def test_no_warn_on_absolute_path(self):
        path = os.path.join(os.path.dirname(App.__file__),
                            'www','zopelogo.jpg')
        App.ImageFile.ImageFile(path)
        self.assertFalse(self.warningshook.warnings)

    def test_no_warn_on_path_as_prefix(self):
        prefix = os.path.dirname(App.__file__)
        App.ImageFile.ImageFile('www/zopelogo.jpg', prefix)
        self.assertFalse(self.warningshook.warnings)

    def test_no_warn_on_namespace_as_prefix(self):
        prefix = App.__dict__ # same as calling globals() inside the App module
        App.ImageFile.ImageFile('www/zopelogo.jpg', prefix)
        self.assertFalse(self.warningshook.warnings)

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(TestImageFile),
        ))



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
'''Zope registerable permissions
'''


from AccessControl.class_init import InitializeClass
from AccessControl.SecurityInfo import ClassSecurityInfo
from Acquisition import Implicit
from OFS.role import RoleManager
from OFS.SimpleItem import Item
from Persistence import Persistent

class Permission(RoleManager,
                 Persistent,
                 Implicit,
                 Item
                ):
    """Model Permission meta-data
    """
    meta_type = 'Zope Permission'
    icon = 'p_/Permission_icon'
    index_html = None
    security = ClassSecurityInfo()

    manage_options=(
        RoleManager.manage_options
        + Item.manage_options
        )

    def __init__(self, id, title, name):
        self.id=id
        self.title=title
        self.name=name

InitializeClass(Permission)



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
"""
Zope signal handlers for clean shutdown, restart and log rotation.
"""

import logging
import os
import sys

import Lifetime

from .threads import dump_threads


logger = logging.getLogger("Z2")

if os.name == 'nt':
    try:
        from WinSignalHandler import SignalHandler
    except ImportError:
        msg = ('Can not install signal handlers.  Please install '
               '(or upgrade) your pywin32 installation '
               '(https://sf.net/projects/pywin32)')
        logger.warning(msg)
        SignalHandler = None
else:
    from SignalHandler import SignalHandler

def shutdownFastHandler():
    """Shutdown cleanly on SIGTERM. This is registered first,
       so it should be called after all other handlers."""
    logger.info("Shutting down fast")
    Lifetime.shutdown(0,fast=1)


def shutdownHandler():
    """Shutdown cleanly on SIGINT. This is registered first,
       so it should be called after all other handlers."""
    logger.info("Shutting down")
    sys.exit(0)

def restartHandler():
    """Restart cleanly on SIGHUP. This is registered first, so it
       should be called after all other SIGHUP handlers."""
    logger.info("Restarting")
    Lifetime.shutdown(1)


def showStacks():
    """Dump a stracktrace of all threads on the console."""
    print dump_threads()
    sys.stdout.flush()


class LogfileReopenHandler:
    """Reopen log files on SIGUSR2.

    This is registered first, so it should be called after all other
    SIGUSR2 handlers.
    """
    def __init__(self, loggers):
        self.loggers = [log for log in loggers if log is not None]

    def __call__(self):
        for log in self.loggers:
            log.reopen()
        logger.info("Log files reopened successfully")

# On Windows, a 'reopen' is useless - the file can not be renamed
# while open, so we perform a trivial 'rotate'.
class LogfileRotateHandler:
    """Rotate log files on SIGUSR2. Only called on Windows. This is 
       registered first, so it should be called after all other SIGUSR2 
       handlers."""
    def __init__(self, loggers):
        self.loggers = [log for log in loggers if log is not None]

    def __call__(self):
        logger.debug("Log files rotation starting...")
        for log in self.loggers:
            for f in log.handler_factories:
                handler = f()
                if hasattr(handler, 'rotate') and callable(handler.rotate):
                    handler.rotate()
        logger.info("Log files rotation complete")

def registerZopeSignals(loggers):
    from signal import SIGTERM, SIGINT
    try:
        from signal import SIGHUP, SIGUSR1, SIGUSR2
    except ImportError:
        # Windows doesn't have these (but also doesn't care what the exact
        # numbers are)
        SIGHUP = 1
        SIGUSR1 = 10
        SIGUSR2 = 12

    if not SignalHandler:
        return

    mod_wsgi = True
    try:
        from mod_wsgi import version
    except ImportError:
        mod_wsgi = False

    if not mod_wsgi:
        SignalHandler.registerHandler(SIGTERM, shutdownFastHandler)
        SignalHandler.registerHandler(SIGINT, shutdownHandler)
    if os.name != 'nt':
        if not mod_wsgi:
            SignalHandler.registerHandler(SIGHUP, restartHandler)
            SignalHandler.registerHandler(SIGUSR1, showStacks)
        SignalHandler.registerHandler(SIGUSR2, LogfileReopenHandler(loggers))
    else:
        # no restart handler on windows.
        # Log files get 'rotated', not 'reopened'
        SignalHandler.registerHandler(SIGUSR2, LogfileRotateHandler(loggers))



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
"""Undo support.
"""

from Acquisition import aq_inner
from Acquisition import aq_parent
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from AccessControl.class_init import InitializeClass
from AccessControl.Permissions import undo_changes
from App.interfaces import IUndoSupport
from App.special_dtml import DTMLFile
from DateTime.DateTime import DateTime
import ExtensionClass
import transaction
from ZopeUndo.Prefix import Prefix
from zope.interface import implements


class UndoSupport(ExtensionClass.Base):

    implements(IUndoSupport)

    security = ClassSecurityInfo()

    manage_options=(
        {'label': 'Undo', 'action': 'manage_UndoForm'},
        )

    security.declareProtected(undo_changes, 'manage_UndoForm')
    manage_UndoForm = DTMLFile(
        'dtml/undo',
        globals(),
        PrincipiaUndoBatchSize=20,
        first_transaction=0,
        last_transaction=20,
        )

    def _get_request_var_or_attr(self, name, default):
        if hasattr(self, 'REQUEST'):
            REQUEST=self.REQUEST
            if REQUEST.has_key(name):
                return REQUEST[name]
            if hasattr(self, name):
                v = getattr(self, name)
            else:
                v = default
            REQUEST[name] = v
            return v
        else:
            if hasattr(self, name):
                v = getattr(self, name)
            else:
                v = default
            return v

    security.declareProtected(undo_changes, 'undoable_transactions')
    def undoable_transactions(self, first_transaction=None,
                              last_transaction=None,
                              PrincipiaUndoBatchSize=None):

        if first_transaction is None:
            first_transaction = self._get_request_var_or_attr(
                'first_transaction', 0)

        if PrincipiaUndoBatchSize is None:
            PrincipiaUndoBatchSize = self._get_request_var_or_attr(
                'PrincipiaUndoBatchSize', 20)

        if last_transaction is None:
            last_transaction = self._get_request_var_or_attr(
                'last_transaction',
                first_transaction+PrincipiaUndoBatchSize)

        spec = {}

        # A user is allowed to undo transactions that were initiated
        # by any member of a user folder in the place where the user
        # is defined.
        user = getSecurityManager().getUser()
        user_parent = aq_parent(user)
        if user_parent is not None:
            path = '/'.join(user_parent.getPhysicalPath()[1:-1])
        else:
            path = ''
        if path:
            spec['user_name'] = Prefix(path)

        if getattr(aq_parent(aq_inner(self)), '_p_jar', None) == self._p_jar:
            # We only want to undo things done here (and not in mounted
            # databases)
            opath = '/'.join(self.getPhysicalPath())
        else:
            # Special case: at the root of a database,
            # allow undo of any path.
            opath = None
        if opath:
            spec['description'] = Prefix(opath)

        r = self._p_jar.db().undoInfo(
            first_transaction, last_transaction, spec)

        for d in r:
            d['time'] = t = DateTime(d['time'])
            desc = d['description']
            tid = d['id']
            if desc:
                desc = desc.split()
                d1 = desc[0]
                desc = ' '.join(desc[1:])
                if len(desc) > 60:
                    desc = desc[:56] + ' ...'
                tid = "%s %s %s %s" % (encode64(tid), t, d1, desc)
            else:
                tid = "%s %s" % (encode64(tid), t)
            d['id'] = tid

        return r

    security.declareProtected(undo_changes, 'manage_undo_transactions')
    def manage_undo_transactions(self, transaction_info=(), REQUEST=None):
        """
        """
        tids = []
        descriptions = []
        for tid in transaction_info:
            tid = tid.split()
            if tid:
                tids.append(decode64(tid[0]))
                descriptions.append(tid[-1])

        if tids:
            transaction.get().note("Undo %s" % ' '.join(descriptions))
            self._p_jar.db().undoMultiple(tids)

        if REQUEST is None:
            return
        REQUEST['RESPONSE'].redirect("%s/manage_UndoForm" % REQUEST['URL1'])
        return ''

InitializeClass(UndoSupport)

########################################################################
# Blech, need this cause binascii.b2a_base64 is too pickly

import binascii


def encode64(s, b2a=binascii.b2a_base64):
    if len(s) < 58:
        return b2a(s)
    r = []
    a = r.append
    for i in range(0, len(s), 57):
        a(b2a(s[i:i+57])[:-1])
    return ''.join(r)


def decode64(s, a2b=binascii.a2b_base64):
    __traceback_info__=len(s), `s`
    return a2b(s+'\n')

del binascii



import sys
import thread
import traceback
import time
from cStringIO import StringIO


def dump_threads():
    """Dump running threads. Returns a string with the tracebacks."""

    frames = sys._current_frames()
    this_thread_id = thread.get_ident()
    now = time.strftime("%Y-%m-%d %H:%M:%S")
    res = ["Threads traceback dump at %s\n" % now]
    for thread_id, frame in frames.iteritems():
        if thread_id == this_thread_id:
            continue

        # Find request in frame
        reqinfo = ''
        f = frame
        while f is not None:
            co = f.f_code
            if co.co_name == 'publish':
                if co.co_filename.endswith('/ZPublisher/Publish.py'):
                    request = f.f_locals.get('request')
                    if request is not None:
                        reqinfo += (request.get('REQUEST_METHOD', '') + ' ' +
                                   request.get('PATH_INFO', ''))
                        qs = request.get('QUERY_STRING')
                        if qs:
                            reqinfo += '?'+qs
                    break
            f = f.f_back
        if reqinfo:
            reqinfo = " (%s)" % reqinfo

        output = StringIO()
        traceback.print_stack(frame, file=output)
        res.append("Thread %s%s:\n%s" %
            (thread_id, reqinfo, output.getvalue()))

    frames = None
    res.append("End of dump")
    return '\n'.join(res)
